#
#   pmacct (Promiscuous mode IP Accounting package)
#   pmacct is Copyright (C) 2003-2020 by Paolo Lucente
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
#   pmgrpcd and its components are Copyright (C) 2018-2020 by:
#
#   Matthias Arnold <matthias.arnold@swisscom.com>
#   Raphaël P. Barazzutti <raphael@barazzutti.net>
#   Juan Camilo Cardona <jccardona82@gmail.com>
#   Thomas Graf <thomas.graf@swisscom.com>
#   Paolo Lucente <paolo@pmacct.net>
#
from confluent_kafka import Producer


p = Producer({"bootstrap.servers": "kafka.sbd.corproot.net:9093"})


def delivery_report(err, msg):
    """ Called once for each message produced to indicate delivery result.
        Triggered by poll() or flush(). """
    if err is not None:
        print("Message delivery failed: {}".format(err))
    else:
        print("Message delivered to {} [{}]".format(msg.topic(), msg.partition()))


for data in ["test"]:
    # Trigger any available delivery report callbacks from previous produce() calls
    p.poll(0)

    # Asynchronously produce a message, the delivery report callback
    # will be triggered from poll() above, or flush() below, when the message has
    # been successfully delivered or failed permanently.
    p.produce("telemetry1.test", data.encode("utf-8"), callback=delivery_report)

# Wait for any outstanding messages to be delivered and delivery report
# callbacks to be triggered.
p.flush()



#
#   pmacct (Promiscuous mode IP Accounting package)
#   pmacct is Copyright (C) 2003-2020 by Paolo Lucente
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
#   pmgrpcd and its components are Copyright (C) 2018-2020 by:
#
#   Matthias Arnold <matthias.arnold@swisscom.com>
#   Raphaël P. Barazzutti <raphael@barazzutti.net>
#   Juan Camilo Cardona <jccardona82@gmail.com>
#   Thomas Graf <thomas.graf@swisscom.com>
#   Paolo Lucente <paolo@pmacct.net>
#
import lib_pmgrpcd
from export_pmgrpcd import Exporter
import ujson as json
import os
from confluent_kafka import Producer
import pickle
import itertools

FOLDER_PICKLE = "/tmp"

class KafkaExporter(Exporter):
    def __init__(self, servers, topic):
        self.producer = Producer({"bootstrap.servers": servers})
        self.topic = topic
        self.encoding_paths_lists = {}
        self.names_data = {}
        for fn in os.listdir(FOLDER_PICKLE):
            if ".pickle" in fn:
                fn = FOLDER_PICKLE + "/" + fn
                with open(fn, 'rb') as fh:
                    data = pickle.load(fh)
                path = data["path"]
                self.names_data[path] = data

    def process_metric(self, datajsonstring):
        jsondata = json.loads(datajsonstring)

        if lib_pmgrpcd.OPTIONS.flatten:
        	self.flatten_pmgrpcd(jsondata)
        else:
	        json_data = json.encode(jsondata)
	        self.send(json_data, self.topic)

    def flatten_pmgrpcd(self, jsondata):
        """
        Extracts the required information from a pmgrpcd 
        json file, and returns a simple one layer dictionary structure.
        """
        source_grpc = jsondata["collector"]["grpc"]["grpcPeer"]
        collection_start = jsondata["collector"]["data"]["collection_timestamp"]
        encoding_path = jsondata["collector"]["data"]["encoding_path"]
        # TODO: Fix this, encoding path includes the proto
        (proto, path) = encoding_path.split(":")
        node_id = jsondata["collector"]["data"]["node_id_str"]

        data = None
        if path in jsondata:
            data = jsondata[path]
        elif "dataGpbkv" in jsondata["collector"]["data"]:
            data = jsondata["collector"]["data"]["dataGpbkv"]

        if data is None:
            raise Exception("We could not extract data")

        if "qos" in encoding_path:
            try:
                flat_data = self.flatten_cisco_fields(data, encoding_path)
                topic = create_topic(encoding_path)
                for metric in flat_data:
                    metric["collection_start"] = collection_start
                    metric["node_id"] = node_id
                    json_data = json.encode(metric)
                    self.send(json_data, topic)
            except Exception as e:
                breakpoint()

        if (
            encoding_path
            != "Cisco-IOS-XR-infra-statsd-oper:infra-statistics/interfaces/interface/latest/generic-counters"
        ):
            return
        try:
            flat_data = self.flatten_cisco_fields(data, encoding_path)
        except:
            breakpoint()

        for metric in flat_data:
            metric["collection_start"] = collection_start
            metric["node_id"] = node_id
            json_data = json.encode(metric)
            self.send(json_data)

    def send(self, text, topic=None):
        if topic is None:
            topic = self.topic
        self.producer.poll(0)
        self.producer.produce(topic, text.encode("utf-8"))

    def flatten_cisco_fields(self, fields, encoding_path, flatten_array=None):
        """
        Takes data and returns one or more flatten jsons.
        """
        if flatten_array is None:
            flatten_array = []
        # let's take care of the key-content type of cisco
        if self.is_key_value(fields):
            self.flatten_key_content(fields, encoding_path, flatten_array)
        else:
            # we might have multiple keys, let's just take them one by one
            for field in fields:
                self.flatten_cisco_fields(field, encoding_path, flatten_array=flatten_array)
                breakpoint()
        return flatten_array

    def flatten_key_content(self, fields, encoding_path, metrics):
        # get keys and content
        keys = None
        content = None
        for field in fields:
            if field.get("name", "") == "keys":
                keys = field
            elif field.get("name", "") == "content":
                content = field
        if keys is None:
            raise Exception("No keys in field {}".format(fields))
        if content is None:
            raise Exception("No content in field {}".format(fields))
        metric_keys = {}
        # flatten keys
        for n, key in enumerate(keys["fields"]):
            key, value = self.simplify_cisco_field(key, encoding_path=encoding_path, key_n=n)
            metric_keys[key] = value
        flatten_metrics = []
        self.flatten_content_fields(content, encoding_path, metric_keys, flatten_metrics, metrics)
        # now we can create the multiple metrics from a single one, if needed
        #for content_f in content["fields"]:
            #if "fields" in content_f and content_f["fields"]:
            #    breakpoint()
            #    raise Exception("Not ready")
            #key, value = simplify_cisco_field(content_f)
            #flatten_metric[key] = value
        metrics.extend(combine_keys_content(metric_keys, flatten_metrics))
        return metrics

    def flatten_content_fields(self, content_f, encoding_path, keys, flatten_metrics, other_metrics, level=None):
        '''
        Here we have pure content.
        '''
        if level is None:
            level = []
        # first we go over elements colleting all "normal" in this hierarchy
        fields_with_children = []
        this_encoding_path = form_encoding_path(encoding_path, level)
        look_for_keys = this_encoding_path in self.encoding_paths_lists
        flatten_metric = {}
        for field in content_f["fields"]:
            if "fields" in field and field["fields"]:
                fields_with_children.append(field)
                continue
            name, value = self.simplify_cisco_field(field, encoding_path=encoding_path, levels=level)
            if look_for_keys:
                if name in self.encoding_paths_lists[this_encoding_path]:
                    keys[name] = value
                else:
                    flatten_metric[name] = value
            else:
                flatten_metric[name] = value

        children_flatten_metrics = []
        if fields_with_children:
            for field in fields_with_children:
                name = field.get("name", None)
                if name is None:
                    name = "Unknown"
                new_levels = level + [name]
                if add_leaf(this_encoding_path, name) in self.encoding_paths_lists:
                    raise Exception("Not ready")
                    new_keys = dict(keys)
                    child_flatten_content = []
                    self.flatten_content_fields(field, this_encoding_path, new_keys, child_flatten_content, other_metrics)
                    # now gett the flatten value and add it
                    new_metric = combine_keys_content(new_keys, child_flatten_content)
                    other_metrics.append(new_metric)
                else:
                    self.flatten_content_fields(field, encoding_path, keys, children_flatten_metrics, other_metrics, new_levels)
                    # our metrics are the ones in chilren_flatten_metrics together with the ones in this hierarchy
        if children_flatten_metrics:
            for children_metric in children_flatten_metrics:
                this_metric = dict(flatten_metric)
                this_metric.update(children_metric)
                flatten_metrics.append(this_metric)
        else:
            flatten_metrics.append(flatten_metric)


    @staticmethod
    def is_key_value(fields):
        return len(fields) == 2 and set(
            [field.get("name", "") for field in fields]
        ) == set(["keys", "content"])

    def simplify_cisco_field(self, field, encoding_path=None, levels=None, key_n=None):
        # find the name, this becomes more of a problem when the mapping is complicated 
        name = None
        if encoding_path in self.names_data:
            if key_n is not None:
                name = self.names_data[encoding_path]["names"][0][key_n]
            else:
                # we are in ocntent
                #relative_path = "/".join(["", *levels, field["name"]])
                #if relative_path in self.names_data[encoding_path]["names"][1]:
                #    name = self.names_data[encoding_path]["names"][1][relative_path]
                name = "_".join([*levels, field["name"]])

        if name is None:
            # problem, log
            name = field["name"]
        value = None
        found = False
        found_attr = None
        for attr in field:
            if attr in ONE_OF:
                found_attr = attr
                found = True
                value = field[attr]
                break
        # try to cast value
        castin = None
        if found_attr in INTEGERS:
            casting = int
        elif found_attr in FLOAT:
            casting = float
        elif found_attr == "boolValue":
            casting = bool
        try:
            value = casting(value)
        except:
            pass
        if not found:
            raise Exception("We could not find a way of simplifying {}".format(field))
        return name, value


def create_topic(path):
    replacesments = set([':', '/'])
    rpath = path
    for ch in replacesments:
        rpath = rpath.replace(ch, ".")
    return rpath

def form_encoding_path(encoding_path, levels):
    if not levels:
        return encoding_path
    if encoding_path[-1] == "/":
        encoding_path = encoding_path[:-1]
    return '/'.join([encoding_path, '/'.join(levels)])

def add_leaf(encoding_path, name):
    if encoding_path[-1] == "/":
        encoding_path = encoding_path[:-1]
    return '/'.join([encoding_path, name])

def combine_keys_content(keys, content):
    # keys are a dict, content is a dict -> list of dicts
    combined = []
    for content_metric  in content:
        metric = dict(keys)
        metric.update(content_metric)
        combined.append(metric)
    return combined

    #for comb in itertools.product(*content.values()):
    #    metric = dict(keys)
    #    for subhierarchy in comb:
    #        metric.update(subhierarchy)
    #    yield metric

    #combined = {}
    #combined.update(keys)
    #combined.update(content)
    #return combined


ONE_OF = set(
    [
        "bytesValue",
        "stringValue",
        "boolValue",
        "uint32Value",
        "uint64Value",
        "sint32Value",
        "sint64Value",
        "doubleValue",
        "floatValue",
    ]
)
INTEGERS = set(["uint32Value", "uint64Value", "sint32Value", "sint64Value"])
FLOAT = set(["doubleValue", "floatValue"])





# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: cisco_telemetry.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='cisco_telemetry.proto',
  package='telemetry',
  syntax='proto3',
  serialized_pb=_b('\n\x15\x63isco_telemetry.proto\x12\ttelemetry\"\xbc\x02\n\tTelemetry\x12\x15\n\x0bnode_id_str\x18\x01 \x01(\tH\x00\x12\x1d\n\x13subscription_id_str\x18\x03 \x01(\tH\x01\x12\x15\n\rencoding_path\x18\x06 \x01(\t\x12\x15\n\rcollection_id\x18\x08 \x01(\x04\x12\x1d\n\x15\x63ollection_start_time\x18\t \x01(\x04\x12\x15\n\rmsg_timestamp\x18\n \x01(\x04\x12-\n\ndata_gpbkv\x18\x0b \x03(\x0b\x32\x19.telemetry.TelemetryField\x12.\n\x08\x64\x61ta_gpb\x18\x0c \x01(\x0b\x32\x1c.telemetry.TelemetryGPBTable\x12\x1b\n\x13\x63ollection_end_time\x18\r \x01(\x04\x42\t\n\x07node_idB\x0e\n\x0csubscription\"\xc1\x02\n\x0eTelemetryField\x12\x11\n\ttimestamp\x18\x01 \x01(\x04\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x15\n\x0b\x62ytes_value\x18\x04 \x01(\x0cH\x00\x12\x16\n\x0cstring_value\x18\x05 \x01(\tH\x00\x12\x14\n\nbool_value\x18\x06 \x01(\x08H\x00\x12\x16\n\x0cuint32_value\x18\x07 \x01(\rH\x00\x12\x16\n\x0cuint64_value\x18\x08 \x01(\x04H\x00\x12\x16\n\x0csint32_value\x18\t \x01(\x11H\x00\x12\x16\n\x0csint64_value\x18\n \x01(\x12H\x00\x12\x16\n\x0c\x64ouble_value\x18\x0b \x01(\x01H\x00\x12\x15\n\x0b\x66loat_value\x18\x0c \x01(\x02H\x00\x12)\n\x06\x66ields\x18\x0f \x03(\x0b\x32\x19.telemetry.TelemetryFieldB\x0f\n\rvalue_by_type\"<\n\x11TelemetryGPBTable\x12\'\n\x03row\x18\x01 \x03(\x0b\x32\x1a.telemetry.TelemetryRowGPB\"C\n\x0fTelemetryRowGPB\x12\x11\n\ttimestamp\x18\x01 \x01(\x04\x12\x0c\n\x04keys\x18\n \x01(\x0c\x12\x0f\n\x07\x63ontent\x18\x0b \x01(\x0c\x62\x06proto3')
)
_sym_db.RegisterFileDescriptor(DESCRIPTOR)




_TELEMETRY = _descriptor.Descriptor(
  name='Telemetry',
  full_name='telemetry.Telemetry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='node_id_str', full_name='telemetry.Telemetry.node_id_str', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='subscription_id_str', full_name='telemetry.Telemetry.subscription_id_str', index=1,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='encoding_path', full_name='telemetry.Telemetry.encoding_path', index=2,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='collection_id', full_name='telemetry.Telemetry.collection_id', index=3,
      number=8, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='collection_start_time', full_name='telemetry.Telemetry.collection_start_time', index=4,
      number=9, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='msg_timestamp', full_name='telemetry.Telemetry.msg_timestamp', index=5,
      number=10, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='data_gpbkv', full_name='telemetry.Telemetry.data_gpbkv', index=6,
      number=11, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='data_gpb', full_name='telemetry.Telemetry.data_gpb', index=7,
      number=12, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='collection_end_time', full_name='telemetry.Telemetry.collection_end_time', index=8,
      number=13, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='node_id', full_name='telemetry.Telemetry.node_id',
      index=0, containing_type=None, fields=[]),
    _descriptor.OneofDescriptor(
      name='subscription', full_name='telemetry.Telemetry.subscription',
      index=1, containing_type=None, fields=[]),
  ],
  serialized_start=37,
  serialized_end=353,
)


_TELEMETRYFIELD = _descriptor.Descriptor(
  name='TelemetryField',
  full_name='telemetry.TelemetryField',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='telemetry.TelemetryField.timestamp', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='name', full_name='telemetry.TelemetryField.name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='bytes_value', full_name='telemetry.TelemetryField.bytes_value', index=2,
      number=4, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='string_value', full_name='telemetry.TelemetryField.string_value', index=3,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='bool_value', full_name='telemetry.TelemetryField.bool_value', index=4,
      number=6, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='uint32_value', full_name='telemetry.TelemetryField.uint32_value', index=5,
      number=7, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='uint64_value', full_name='telemetry.TelemetryField.uint64_value', index=6,
      number=8, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='sint32_value', full_name='telemetry.TelemetryField.sint32_value', index=7,
      number=9, type=17, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='sint64_value', full_name='telemetry.TelemetryField.sint64_value', index=8,
      number=10, type=18, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='double_value', full_name='telemetry.TelemetryField.double_value', index=9,
      number=11, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='float_value', full_name='telemetry.TelemetryField.float_value', index=10,
      number=12, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='fields', full_name='telemetry.TelemetryField.fields', index=11,
      number=15, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='value_by_type', full_name='telemetry.TelemetryField.value_by_type',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=356,
  serialized_end=677,
)


_TELEMETRYGPBTABLE = _descriptor.Descriptor(
  name='TelemetryGPBTable',
  full_name='telemetry.TelemetryGPBTable',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='row', full_name='telemetry.TelemetryGPBTable.row', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=679,
  serialized_end=739,
)


_TELEMETRYROWGPB = _descriptor.Descriptor(
  name='TelemetryRowGPB',
  full_name='telemetry.TelemetryRowGPB',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='telemetry.TelemetryRowGPB.timestamp', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='keys', full_name='telemetry.TelemetryRowGPB.keys', index=1,
      number=10, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='content', full_name='telemetry.TelemetryRowGPB.content', index=2,
      number=11, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=741,
  serialized_end=808,
)

_TELEMETRY.fields_by_name['data_gpbkv'].message_type = _TELEMETRYFIELD
_TELEMETRY.fields_by_name['data_gpb'].message_type = _TELEMETRYGPBTABLE
_TELEMETRY.oneofs_by_name['node_id'].fields.append(
  _TELEMETRY.fields_by_name['node_id_str'])
_TELEMETRY.fields_by_name['node_id_str'].containing_oneof = _TELEMETRY.oneofs_by_name['node_id']
_TELEMETRY.oneofs_by_name['subscription'].fields.append(
  _TELEMETRY.fields_by_name['subscription_id_str'])
_TELEMETRY.fields_by_name['subscription_id_str'].containing_oneof = _TELEMETRY.oneofs_by_name['subscription']
_TELEMETRYFIELD.fields_by_name['fields'].message_type = _TELEMETRYFIELD
_TELEMETRYFIELD.oneofs_by_name['value_by_type'].fields.append(
  _TELEMETRYFIELD.fields_by_name['bytes_value'])
_TELEMETRYFIELD.fields_by_name['bytes_value'].containing_oneof = _TELEMETRYFIELD.oneofs_by_name['value_by_type']
_TELEMETRYFIELD.oneofs_by_name['value_by_type'].fields.append(
  _TELEMETRYFIELD.fields_by_name['string_value'])
_TELEMETRYFIELD.fields_by_name['string_value'].containing_oneof = _TELEMETRYFIELD.oneofs_by_name['value_by_type']
_TELEMETRYFIELD.oneofs_by_name['value_by_type'].fields.append(
  _TELEMETRYFIELD.fields_by_name['bool_value'])
_TELEMETRYFIELD.fields_by_name['bool_value'].containing_oneof = _TELEMETRYFIELD.oneofs_by_name['value_by_type']
_TELEMETRYFIELD.oneofs_by_name['value_by_type'].fields.append(
  _TELEMETRYFIELD.fields_by_name['uint32_value'])
_TELEMETRYFIELD.fields_by_name['uint32_value'].containing_oneof = _TELEMETRYFIELD.oneofs_by_name['value_by_type']
_TELEMETRYFIELD.oneofs_by_name['value_by_type'].fields.append(
  _TELEMETRYFIELD.fields_by_name['uint64_value'])
_TELEMETRYFIELD.fields_by_name['uint64_value'].containing_oneof = _TELEMETRYFIELD.oneofs_by_name['value_by_type']
_TELEMETRYFIELD.oneofs_by_name['value_by_type'].fields.append(
  _TELEMETRYFIELD.fields_by_name['sint32_value'])
_TELEMETRYFIELD.fields_by_name['sint32_value'].containing_oneof = _TELEMETRYFIELD.oneofs_by_name['value_by_type']
_TELEMETRYFIELD.oneofs_by_name['value_by_type'].fields.append(
  _TELEMETRYFIELD.fields_by_name['sint64_value'])
_TELEMETRYFIELD.fields_by_name['sint64_value'].containing_oneof = _TELEMETRYFIELD.oneofs_by_name['value_by_type']
_TELEMETRYFIELD.oneofs_by_name['value_by_type'].fields.append(
  _TELEMETRYFIELD.fields_by_name['double_value'])
_TELEMETRYFIELD.fields_by_name['double_value'].containing_oneof = _TELEMETRYFIELD.oneofs_by_name['value_by_type']
_TELEMETRYFIELD.oneofs_by_name['value_by_type'].fields.append(
  _TELEMETRYFIELD.fields_by_name['float_value'])
_TELEMETRYFIELD.fields_by_name['float_value'].containing_oneof = _TELEMETRYFIELD.oneofs_by_name['value_by_type']
_TELEMETRYGPBTABLE.fields_by_name['row'].message_type = _TELEMETRYROWGPB
DESCRIPTOR.message_types_by_name['Telemetry'] = _TELEMETRY
DESCRIPTOR.message_types_by_name['TelemetryField'] = _TELEMETRYFIELD
DESCRIPTOR.message_types_by_name['TelemetryGPBTable'] = _TELEMETRYGPBTABLE
DESCRIPTOR.message_types_by_name['TelemetryRowGPB'] = _TELEMETRYROWGPB

Telemetry = _reflection.GeneratedProtocolMessageType('Telemetry', (_message.Message,), dict(
  DESCRIPTOR = _TELEMETRY,
  __module__ = 'cisco_telemetry_pb2'
  # @@protoc_insertion_point(class_scope:telemetry.Telemetry)
  ))
_sym_db.RegisterMessage(Telemetry)

TelemetryField = _reflection.GeneratedProtocolMessageType('TelemetryField', (_message.Message,), dict(
  DESCRIPTOR = _TELEMETRYFIELD,
  __module__ = 'cisco_telemetry_pb2'
  # @@protoc_insertion_point(class_scope:telemetry.TelemetryField)
  ))
_sym_db.RegisterMessage(TelemetryField)

TelemetryGPBTable = _reflection.GeneratedProtocolMessageType('TelemetryGPBTable', (_message.Message,), dict(
  DESCRIPTOR = _TELEMETRYGPBTABLE,
  __module__ = 'cisco_telemetry_pb2'
  # @@protoc_insertion_point(class_scope:telemetry.TelemetryGPBTable)
  ))
_sym_db.RegisterMessage(TelemetryGPBTable)

TelemetryRowGPB = _reflection.GeneratedProtocolMessageType('TelemetryRowGPB', (_message.Message,), dict(
  DESCRIPTOR = _TELEMETRYROWGPB,
  __module__ = 'cisco_telemetry_pb2'
  # @@protoc_insertion_point(class_scope:telemetry.TelemetryRowGPB)
  ))
_sym_db.RegisterMessage(TelemetryRowGPB)


# @@protoc_insertion_point(module_scope)



#
#   pmacct (Promiscuous mode IP Accounting package)
#   pmacct is Copyright (C) 2003-2020 by Paolo Lucente
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
#   pmgrpcd and its components are Copyright (C) 2018-2020 by:
#
#   Matthias Arnold <matthias.arnold@swisscom.com>
#   Raphaël P. Barazzutti <raphael@barazzutti.net>
#   Juan Camilo Cardona <jccardona82@gmail.com>
#   Thomas Graf <thomas.graf@swisscom.com>
#   Paolo Lucente <paolo@pmacct.net>
#
#
from datetime import datetime
import pprint
import json

global mitigation
mitigation = {}


def mod_all_json_data(resdict):
    global mitigation

    mitigation = resdict.copy()

    if "collector" in mitigation:
        if ("grpc" in mitigation["collector"]) and ("data" in mitigation["collector"]):
            if "ne_vendor" in mitigation["collector"]["grpc"]:
                mod_all_pre()
                if mitigation["collector"]["grpc"]["ne_vendor"] == "Huawei":
                    mod_huawei()
                elif mitigation["collector"]["grpc"]["ne_vendor"] == "Cisco":
                    mod_cisco()
                mod_all_post()
    return mitigation


def mod_all_pre():
    global mitigation

    # especially Cisco oc_int has to have a Array after the "subinterface" but anyway it is good for all vendors in case it will be without array
    addanywayarray_subint()

    # Cisco oc_int has to have a Array after the "interface" but anyway it is good for all vendors in case it will be without array
    addanywayarray_int()

    # CHANGE ON EACH KEY THE DASH TO UNDERSCORE because avro only support underscore
    obj = mitigation.copy()
    mitigation = d2u(obj)


def mod_huawei():
    global mitigation
    # rename Huawei sensor_path to encoding_path because it has to match with the avroschema
    sens2enco()

    # "interface":[
    # The parent  container "interfaces" is missing for "interface" in case of Huawei
    # https://github.com/openconfig/public/blob/master/release/models/interfaces/openconfig-interfaces.yang#L1012
    insert_rec_interfaces()

    # "admin_status":0,
    # "oper_status":0,
    # Huawei oc_interface "admin_status" and "oper_status" contains 0 or 1 but has to have "UP" or "DOWN"
    # https://github.com/openconfig/public/blob/master/release/models/interfaces/openconfig-interfaces.yang#L464
    # https://github.com/openconfig/public/blob/master/release/models/interfaces/openconfig-interfaces.yang#L496
    # because Huawei told us that the the hole section state is not in "acquisition path" of openconfig-interfaces i will remove all the leaves of "state".
    # only the section "counters" within the section "state" will be there
    remove_hua_state_with_ifindex0()

    #leaf admin-status and oper-status are type enumeration. 
    #see https://github.com/openconfig/public.git -> public/release/models/interfaces/openconfig-interfaces.yang
    #"admin_status":1,  -> "admin_status":"UP",
    #"oper_status":2,   -> "oper_status":"DOWN",
    mod_hua_int_admin_and_oper_status_to_enum()
    mod_hua_subint_admin_and_oper_status_to_enum()

    # encoding_path of Hua has to be mach the jsondata:
    # "encoding_path":"openconfig-interfaces:interfaces\/interface\/state\/counters" -> "encoding_path":"openconfig-interfaces:interfaces",
    correct_huaw_enc_path()


def mod_cisco():
    global mitigation
    pass


def mod_all_post():
    global mitigation

    # https://github.com/openconfig/public/blob/master/release/models/interfaces/openconfig-interfaces.yang#L615
    # "last_clear":"",
    # last_clear has to be epoch (integer) not string
    # https://github.com/openconfig/public/blob/master/release/models/interfaces/openconfig-interfaces.yang#L854
    # https://tools.ietf.org/html/rfc6991
    # typedef timeticks {
    #  type uint32;
    #    description
    #    "The timeticks type represents a non-negative integer that
    #     represents the time, modulo 2^32 (4294967296 decimal), in
    #
    # "in_unicast_pkts":"595769",
    #  #oc_interface all counters of oc_int should be long
    mod_int_lastclear()
    mod_subint_lastclear()

    #In general all counter exept the last_clear has to be integer for interfaces and subinterfaces
    int_state_counters_2_integer()
    subint_state_counters_2_integer()

    # All collector meta-data part of collector -> data have to be all the time the correct data-type.
    # independent of the vendor or routertyp.
    # Only defined keys are allowed because of mandatory avroschema
    # In general remove all kv-pairs of record["collector"]["data"] except this ones:
    # "collection_end_time":1548319798771,
    # "collection_id":"3007",
    # "collection_start_time":1548319798741,
    # "encoding_path":"openconfig-interfaces:interfaces\/interface\/state\/counters",
    # "msg_timestamp":1548319798831,
    # "node_id_str":"ipi-zbb900-r-al-01",
    # "subscription_id_str":"DAISY63"
    harmonize_collector_data()

    #for hua
    #leaf last-change is type oc-types:timeticks64;
    #see https://github.com/openconfig/public.git -> public/release/models/interfaces/openconfig-interfaces.yang
    #"last_change":"1234567890", -> "last_change":1234567890,
    #
    #for cisco
    #"last_change":"2019-01-08T12:53:02Z", -> "last_change":1546951982,
    #
    #and also doing this (nothing) for already good cases:
    #"last_change":1234567890, -> "last_change":1234567890,
    mod_int_lastchange()
    mod_subint_lastchange()

# Helper-Methods for ALL
# ---------------------------
def addanywayarray_subint():
    if "encoding_path" in mitigation["collector"]["data"]:
        if (
            mitigation["collector"]["data"]["encoding_path"]
            == "openconfig-interfaces:interfaces"
        ):
            if "interfaces" in mitigation:
                if "interface" in mitigation["interfaces"]:
                    if "subinterfaces" in mitigation["interfaces"]["interface"]:
                        if (
                            "subinterface"
                            in mitigation["interfaces"]["interface"]["subinterfaces"]
                        ):
                            if (
                                type(
                                    mitigation["interfaces"]["interface"][
                                        "subinterfaces"
                                    ]["subinterface"]
                                )
                                == dict
                            ):
                                mitigation["interfaces"]["interface"]["subinterfaces"][
                                    "subinterface"
                                ] = [
                                    mitigation["interfaces"]["interface"][
                                        "subinterfaces"
                                    ]["subinterface"]
                                ]


def addanywayarray_int():
    if "encoding_path" in mitigation["collector"]["data"]:
        if (
            mitigation["collector"]["data"]["encoding_path"]
            == "openconfig-interfaces:interfaces"
        ):
            if "interfaces" in mitigation:
                if "interface" in mitigation["interfaces"]:
                    if type(mitigation["interfaces"]["interface"]) == dict:
                        mitigation["interfaces"]["interface"] = [
                            mitigation["interfaces"]["interface"]
                        ]


def d2u(obj):
    if isinstance(obj, (str, int, float)):
        return obj
    if isinstance(obj, dict):
        new = obj.__class__()
        for k, v in obj.items():
            new[k.replace("-", "_")] = d2u(v)
    elif isinstance(obj, (list, set, tuple)):
        new = obj.__class__(d2u(v) for v in obj)
    else:
        return obj
    return new


def harmonize_collector_data():
    coldata = mitigation["collector"]["data"].copy()
    newcoldata = {}

    inttype = [
        "collection_timestamp",
        "collection_end_time",
        "collection_start_time",
        "msg_timestamp",
    ]
    stringtype = [
        "collection_id",
        "encoding_path",
        "node_id_str",
        "subscription_id_str",
        "encoding_type",
    ]

    for elem in mitigation["collector"]["data"]:
        if elem in inttype:
            newcoldata.update({elem: int(mitigation["collector"]["data"][elem])})
            inttype.remove(elem)
        elif elem in stringtype:
            newcoldata.update({elem: str(mitigation["collector"]["data"][elem])})
            stringtype.remove(elem)
        else:
            pass
    for elem in inttype:
        newcoldata.update({elem: -1})
    for elem in stringtype:
        newcoldata.update({elem: "None"})

    #  newcoldata.update({"collection_timestamp": int(mitigation["collector"]["data"]["collection_timestamp"])})
    #  newcoldata.update({"collection_end_time": int(mitigation["collector"]["data"]["collection_end_time"])})
    #  newcoldata.update({"collection_id": str(mitigation["collector"]["data"]["collection_id"])})
    #  newcoldata.update({"collection_start_time": int(mitigation["collector"]["data"]["collection_start_time"])})
    #  newcoldata.update({"encoding_path": str(mitigation["collector"]["data"]["encoding_path"])})
    #  newcoldata.update({"msg_timestamp": int(mitigation["collector"]["data"]["msg_timestamp"])})
    #  newcoldata.update({"node_id_str": str(mitigation["collector"]["data"]["node_id_str"])})
    #  newcoldata.update({"subscription_id_str": str(mitigation["collector"]["data"]["subscription_id_str"])})

    mitigation["collector"]["data"] = newcoldata


# Helper-Methods for CISCO
# ---------------------------

def timestuff2epoch(lc):
    if isinstance(lc, int):
        lc_mod = lc
    else:
        try:
            # "last_clear":"2019-01-08T12:53:02Z",
            utc_dt = datetime.strptime(lc, "%Y-%m-%dT%H:%M:%SZ")
            timestamp = (utc_dt - datetime(1970, 1, 1)).total_seconds()
        except ValueError:
            try:
                int(lc)
            except ValueError:
                lc_mod = 0 
            else:
                lc_mod = int(lc)
        else:
            lc_mod = int(timestamp)
    return lc_mod

def mod_int_lastclear():
    if "encoding_path" in mitigation["collector"]["data"]:
        if "openconfig-interfaces:" in mitigation["collector"]["data"]["encoding_path"]:
            if "interfaces" in mitigation:
                if "interface" in mitigation["interfaces"]:
                    for intelem in mitigation["interfaces"]["interface"]:
                        intidx = mitigation["interfaces"]["interface"].index(intelem)
                        if "state" in mitigation["interfaces"]["interface"][intidx]:
                            if ("counters" in mitigation["interfaces"]["interface"][intidx]["state"]):
                                for leave in mitigation["interfaces"]["interface"][intidx]["state"]["counters"]:
                                    if leave == "last_clear":
                                        lc = mitigation["interfaces"]["interface"][intidx]["state"]["counters"]["last_clear"]
                                        mitigation["interfaces"]["interface"][intidx]["state"]["counters"]["last_clear"] = timestuff2epoch(lc)
                                    if leave == "last-clear":
                                        lc = mitigation["interfaces"]["interface"][intidx]["state"]["counters"]["last-clear"]
                                        mitigation["interfaces"]["interface"][intidx]["state"]["counters"]["last-clear"] = timestuff2epoch(lc)


def mod_subint_lastclear():
    if "encoding_path" in mitigation["collector"]["data"]:
        if "openconfig-interfaces:" in mitigation["collector"]["data"]["encoding_path"]:
            if "interfaces" in mitigation:
                if "interface" in mitigation["interfaces"]:
                    for intelem in mitigation["interfaces"]["interface"]:
                        intidx = mitigation["interfaces"]["interface"].index(intelem)
                        if ("subinterfaces" in mitigation["interfaces"]["interface"][intidx]):
                            if ("subinterface" in mitigation["interfaces"]["interface"][intidx]["subinterfaces"]):
                                for subintelem in mitigation["interfaces"]["interface"][intidx]["subinterfaces"]["subinterface"]:
                                    subintidx = mitigation["interfaces"]["interface"][intidx]["subinterfaces"]["subinterface"].index(subintelem)
                                    if ("state" in mitigation["interfaces"]["interface"][intidx]["subinterfaces"]["subinterface"][subintidx]):
                                        if ("counters" in mitigation["interfaces"]["interface"][intidx]["subinterfaces"]["subinterface"][subintidx]["state"]):
                                            for leave in mitigation["interfaces"][ "interface" ][intidx]["subinterfaces"]["subinterface"][subintidx]["state"]["counters"]:
                                                if leave == "last_clear":
                                                    lc = mitigation["interfaces"]["interface"][intidx]["subinterfaces"]["subinterface"][subintidx]["state"]["counters"]["last_clear"]
                                                    mitigation["interfaces"]["interface"][intidx]["subinterfaces"]["subinterface"][subintidx]["state"]["counters"]["last_clear"] = timestuff2epoch(lc)
                                                if leave == "last-clear":
                                                    lc = mitigation["interfaces"]["interface"][intidx]["subinterfaces"]["subinterface"][subintidx]["state"]["counters"]["last-clear"]
                                                    mitigation["interfaces"]["interface"][intidx]["subinterfaces"]["subinterface"][subintidx]["state"]["counters"]["last-clear"] = timestuff2epoch(lc)

def mod_int_lastchange():
    if "encoding_path" in mitigation["collector"]["data"]:
        if "openconfig-interfaces:" in mitigation["collector"]["data"]["encoding_path"]:
            if "interfaces" in mitigation:
                if "interface" in mitigation["interfaces"]:
                    for intelem in mitigation["interfaces"]["interface"]:
                        intidx = mitigation["interfaces"]["interface"].index(intelem)
                        if "state" in mitigation["interfaces"]["interface"][intidx]:
                            if ("last_change" in mitigation["interfaces"]["interface"][intidx]["state"]):
                                lc = mitigation["interfaces"]["interface"][intidx]["state"]["last_change"]
                                mitigation["interfaces"]["interface"][intidx]["state"]["last_change"] = timestuff2epoch(lc)
                            if ("last-change" in mitigation["interfaces"]["interface"][intidx]["state"]):
                                lc = mitigation["interfaces"]["interface"][intidx]["state"]["last-change"]
                                mitigation["interfaces"]["interface"][intidx]["state"]["last-change"] = timestuff2epoch(lc)


def mod_subint_lastchange():
    if "encoding_path" in mitigation["collector"]["data"]:
        if "openconfig-interfaces:" in mitigation["collector"]["data"]["encoding_path"]:
            if "interfaces" in mitigation:
                if "interface" in mitigation["interfaces"]:
                    for intelem in mitigation["interfaces"]["interface"]:
                        intidx = mitigation["interfaces"]["interface"].index(intelem)
                        if ("subinterfaces" in mitigation["interfaces"]["interface"][intidx]):
                            if ("subinterface" in mitigation["interfaces"]["interface"][intidx]["subinterfaces"]):
                                for subintelem in mitigation["interfaces"]["interface"][intidx]["subinterfaces"]["subinterface"]:
                                    subintidx = mitigation["interfaces"]["interface"][intidx]["subinterfaces"]["subinterface"].index(subintelem)
                                    if ("state" in mitigation["interfaces"]["interface"][intidx]["subinterfaces"]["subinterface"][subintidx]):
                                        if ("last_change" in mitigation["interfaces"]["interface"][intidx]["subinterfaces"]["subinterface"][subintidx]["state"]):
                                            lc = mitigation["interfaces"]["interface"][intidx]["subinterfaces"]["subinterface"][subintidx]["state"]["last_change"]
                                            mitigation["interfaces"]["interface"][intidx]["subinterfaces"]["subinterface"][subintidx]["state"]["last_change"] = timestuff2epoch(lc)
                                        if ("last-change" in mitigation["interfaces"]["interface"][intidx]["subinterfaces"]["subinterface"][subintidx]["state"]):
                                            lc = mitigation["interfaces"]["interface"][intidx]["subinterfaces"]["subinterface"][subintidx]["state"]["last-change"]
                                            mitigation["interfaces"]["interface"][intidx]["subinterfaces"]["subinterface"][subintidx]["state"]["last-change"] = timestuff2epoch(lc)
                                        


# Helper-Methods for HUAWEI
# ---------------------------
def sens2enco():
    if "collector" in mitigation:
        if "data" in mitigation["collector"]:
            if "sensor_path" in mitigation["collector"]["data"]:
                mitigation["collector"]["data"].update(
                    {"encoding_path": mitigation["collector"]["data"]["sensor_path"]}
                )
                del mitigation["collector"]["data"]["sensor_path"]


def insert_rec_interfaces():
    if "encoding_path" in mitigation["collector"]["data"]:
        if "openconfig-interfaces:" in mitigation["collector"]["data"]["encoding_path"]:
            if "interface" in mitigation:
                if type(mitigation["interface"]) == list:
                    mitigation.update(
                        {"interfaces": {"interface": mitigation["interface"]}}
                    )
                    del mitigation["interface"]


def remove_hua_state_with_ifindex0():
    if "encoding_path" in mitigation["collector"]["data"]:
        if "openconfig-interfaces:" in mitigation["collector"]["data"]["encoding_path"]:
            if "interfaces" in mitigation:
                if "interface" in mitigation["interfaces"]:
                    interface_list = list(mitigation["interfaces"]["interface"])
                    for elem in interface_list:
                        idx = interface_list.index(elem)
                        if "state" in interface_list[idx]:
                            if "ifindex" in interface_list[idx]["state"]:
                                if interface_list[idx]["state"]["ifindex"] == 0:
                                    if "counters" in interface_list[idx]["state"]:
                                        mitigation["interfaces"]["interface"][idx]["state"] = {
                                             "counters": mitigation["interfaces"]["interface"][idx]["state"]["counters"]
                                        }


def mod_hua_int_admin_and_oper_status_to_enum():
    if "encoding_path" in mitigation["collector"]["data"]:
        if "openconfig-interfaces:" in mitigation["collector"]["data"]["encoding_path"]:
            if "interfaces" in mitigation:
                if "interface" in mitigation["interfaces"]:
                    interface_list = list(mitigation["interfaces"]["interface"])
                    for elem in interface_list:
                        idx = interface_list.index(elem)
                        if "state" in interface_list[idx]:
                            if "ifindex" in interface_list[idx]["state"]:
                                if not interface_list[idx]["state"]["ifindex"] == 0:
                                    if "admin_status" in interface_list[idx]["state"]:
                                       if interface_list[idx]["state"]["admin_status"] == 0:
                                          mitigation["interfaces"]["interface"][idx]["state"]["admin_status"] = "INVALID"
                                       if interface_list[idx]["state"]["admin_status"] == 1:
                                          mitigation["interfaces"]["interface"][idx]["state"]["admin_status"] = "UP"
                                       if interface_list[idx]["state"]["admin_status"] == 2:
                                          mitigation["interfaces"]["interface"][idx]["state"]["admin_status"] = "DOWN"
                                       if interface_list[idx]["state"]["admin_status"] == 3:
                                          mitigation["interfaces"]["interface"][idx]["state"]["admin_status"] = "TESTING"
                                    if "oper_status" in interface_list[idx]["state"]:
                                       if interface_list[idx]["state"]["oper_status"] == 0:
                                          mitigation["interfaces"]["interface"][idx]["state"]["oper_status"] = "INVALID"
                                       if interface_list[idx]["state"]["oper_status"] == 1:
                                          mitigation["interfaces"]["interface"][idx]["state"]["oper_status"] = "UP"
                                       if interface_list[idx]["state"]["oper_status"] == 2:
                                          mitigation["interfaces"]["interface"][idx]["state"]["oper_status"] = "DOWN"
                                       if interface_list[idx]["state"]["oper_status"] == 3:
                                          mitigation["interfaces"]["interface"][idx]["state"]["oper_status"] = "TESTING"
                                       if interface_list[idx]["state"]["oper_status"] == 4:
                                          mitigation["interfaces"]["interface"][idx]["state"]["oper_status"] = "UNKNOWN"
                                       if interface_list[idx]["state"]["oper_status"] == 5:
                                          mitigation["interfaces"]["interface"][idx]["state"]["oper_status"] = "DORMANT"
                                       if interface_list[idx]["state"]["oper_status"] == 6:
                                          mitigation["interfaces"]["interface"][idx]["state"]["oper_status"] = "NOT_PRESENT"
                                       if interface_list[idx]["state"]["oper_status"] == 7:
                                          mitigation["interfaces"]["interface"][idx]["state"]["oper_status"] = "LOWER_LAYER_DOWN"

def mod_hua_subint_admin_and_oper_status_to_enum():
    if "encoding_path" in mitigation["collector"]["data"]:
        if "openconfig-interfaces:" in mitigation["collector"]["data"]["encoding_path"]:
            if "interfaces" in mitigation:
                if "interface" in mitigation["interfaces"]:
                    interface_list = list(mitigation["interfaces"]["interface"])
                    for elem in interface_list:
                        idx = interface_list.index(elem)
                        if "subinterfaces" in interface_list[idx]:
                            if "subinterface" in interface_list[idx]["subinterfaces"]:
                                subinterface_list = list(interface_list[idx]["subinterfaces"]["subinterface"])
                                for sub_elem in subinterface_list:
                                    subidx = subinterface_list.index(sub_elem)
                                    if "state" in subinterface_list[subidx]:
                                        if "ifindex" in subinterface_list[subidx]["state"]:
                                            if not subinterface_list[subidx]["state"]["ifindex"] == 0:
                                                if "admin_status" in subinterface_list[subidx]["state"]:
                                                    if subinterface_list[subidx]["state"]["admin_status"] == 0:
                                                       mitigation["interfaces"]["interface"][idx]["subinterfaces"]["subinterface"][subidx]["state"]["admin_status"] = "INVALID"
                                                    if subinterface_list[subidx]["state"]["admin_status"] == 1:
                                                       mitigation["interfaces"]["interface"][idx]["subinterfaces"]["subinterface"][subidx]["state"]["admin_status"] = "UP"
                                                    if subinterface_list[subidx]["state"]["admin_status"] == 2:
                                                       mitigation["interfaces"]["interface"][idx]["subinterfaces"]["subinterface"][subidx]["state"]["admin_status"] = "DOWN"
                                                    if subinterface_list[subidx]["state"]["admin_status"] == 3:
                                                       mitigation["interfaces"]["interface"][idx]["subinterfaces"]["subinterface"][subidx]["state"]["admin_status"] = "TESTING"
                                                if "oper_status" in subinterface_list[subidx]["state"]:
                                                    if subinterface_list[subidx]["state"]["oper_status"] == 0:
                                                       mitigation["interfaces"]["interface"][idx]["subinterfaces"]["subinterface"][subidx]["state"]["oper_status"] = "INVALID"
                                                    if subinterface_list[subidx]["state"]["oper_status"] == 1:
                                                       mitigation["interfaces"]["interface"][idx]["subinterfaces"]["subinterface"][subidx]["state"]["oper_status"] = "UP"
                                                    if subinterface_list[subidx]["state"]["oper_status"] == 2:
                                                       mitigation["interfaces"]["interface"][idx]["subinterfaces"]["subinterface"][subidx]["state"]["oper_status"] = "DOWN"
                                                    if subinterface_list[subidx]["state"]["oper_status"] == 3:
                                                       mitigation["interfaces"]["interface"][idx]["subinterfaces"]["subinterface"][subidx]["state"]["oper_status"] = "TESTING"
                                                    if subinterface_list[subidx]["state"]["oper_status"] == 4:
                                                       mitigation["interfaces"]["interface"][idx]["subinterfaces"]["subinterface"][subidx]["state"]["oper_status"] = "UNKNOWN"
                                                    if subinterface_list[subidx]["state"]["oper_status"] == 5:
                                                       mitigation["interfaces"]["interface"][idx]["subinterfaces"]["subinterface"][subidx]["state"]["oper_status"] = "DORMANT"
                                                    if subinterface_list[subidx]["state"]["oper_status"] == 6:
                                                       mitigation["interfaces"]["interface"][idx]["subinterfaces"]["subinterface"][subidx]["state"]["oper_status"] = "NOT_PRESENT"
                                                    if subinterface_list[subidx]["state"]["oper_status"] == 7:
                                                       mitigation["interfaces"]["interface"][idx]["subinterfaces"]["subinterface"][subidx]["state"]["oper_status"] = "LOWER_LAYER_DOWN"


def int_state_counters_2_integer():
    if "encoding_path" in mitigation["collector"]["data"]:
        if "openconfig-interfaces:" in mitigation["collector"]["data"]["encoding_path"]:
            if "interfaces" in mitigation:
                if "interface" in mitigation["interfaces"]:
                    for elem in mitigation["interfaces"]["interface"]:
                        idx = mitigation["interfaces"]["interface"].index(elem)
                        if "state" in mitigation["interfaces"]["interface"][idx]:
                            if (
                                "counters"
                                in mitigation["interfaces"]["interface"][idx]["state"]
                            ):
                                for leave in mitigation["interfaces"]["interface"][idx][
                                    "state"
                                ]["counters"]:
                                    if not (leave == "last_clear"):
                                        try:
                                            int(
                                                mitigation["interfaces"]["interface"][
                                                    idx
                                                ]["state"]["counters"][leave]
                                            )
                                        except ValueError:
                                            mitigation["interfaces"]["interface"][idx][
                                                "state"
                                            ]["counters"][leave] = -1
                                        else:
                                            mitigation["interfaces"]["interface"][idx][
                                                "state"
                                            ]["counters"][leave] = int(
                                                mitigation["interfaces"]["interface"][
                                                    idx
                                                ]["state"]["counters"][leave]
                                            )
                                        if (
                                            mitigation["interfaces"]["interface"][idx][
                                                "state"
                                            ]["counters"][leave]
                                            > 9223372036854775807
                                        ):
                                            mitigation["interfaces"]["interface"][idx][
                                                "state"
                                            ]["counters"][leave] = (
                                                mitigation["interfaces"]["interface"][
                                                    idx
                                                ]["state"]["counters"][leave]
                                                - 9223372036854775808
                                            )
                                    else:
                                        try:
                                            int(
                                                mitigation["interfaces"]["interface"][
                                                    idx
                                                ]["state"]["counters"][leave]
                                            )
                                        except ValueError:
                                            mitigation["interfaces"]["interface"][idx][
                                                "state"
                                            ]["counters"][leave] = 0
                                        else:
                                            mitigation["interfaces"]["interface"][idx][
                                                "state"
                                            ]["counters"][leave] = int(
                                                mitigation["interfaces"]["interface"][
                                                    idx
                                                ]["state"]["counters"][leave]
                                            )


def subint_state_counters_2_integer():
    if "encoding_path" in mitigation["collector"]["data"]:
        if "openconfig-interfaces:" in mitigation["collector"]["data"]["encoding_path"]:
            if "interfaces" in mitigation:
                if "interface" in mitigation["interfaces"]:
                    for elem in mitigation["interfaces"]["interface"]:
                        idx = mitigation["interfaces"]["interface"].index(elem)
                        if (
                            "subinterfaces"
                            in mitigation["interfaces"]["interface"][idx]
                        ):
                            if (
                                "subinterface"
                                in mitigation["interfaces"]["interface"][idx][
                                    "subinterfaces"
                                ]
                            ):
                                for subelem in mitigation["interfaces"]["interface"][
                                    idx
                                ]["subinterfaces"]["subinterface"]:
                                    subidx = mitigation["interfaces"]["interface"][idx][
                                        "subinterfaces"
                                    ]["subinterface"].index(subelem)
                                    if (
                                        "state"
                                        in mitigation["interfaces"]["interface"][idx][
                                            "subinterfaces"
                                        ]["subinterface"][subidx]
                                    ):
                                        if (
                                            "counters"
                                            in mitigation["interfaces"]["interface"][
                                                idx
                                            ]["subinterfaces"]["subinterface"][subidx][
                                                "state"
                                            ]
                                        ):
                                            for leave in mitigation["interfaces"][
                                                "interface"
                                            ][idx]["subinterfaces"]["subinterface"][
                                                subidx
                                            ][
                                                "state"
                                            ][
                                                "counters"
                                            ]:
                                                if not (leave == "last_clear"):
                                                    try:
                                                        int(
                                                            mitigation["interfaces"][
                                                                "interface"
                                                            ][idx]["subinterfaces"][
                                                                "subinterface"
                                                            ][
                                                                subidx
                                                            ][
                                                                "state"
                                                            ][
                                                                "counters"
                                                            ][
                                                                leave
                                                            ]
                                                        )
                                                    except ValueError:
                                                        mitigation["interfaces"][
                                                            "interface"
                                                        ][idx]["subinterfaces"][
                                                            "subinterface"
                                                        ][
                                                            subidx
                                                        ][
                                                            "state"
                                                        ][
                                                            "counters"
                                                        ][
                                                            leave
                                                        ] = -1
                                                    else:
                                                        mitigation["interfaces"][
                                                            "interface"
                                                        ][idx]["subinterfaces"][
                                                            "subinterface"
                                                        ][
                                                            subidx
                                                        ][
                                                            "state"
                                                        ][
                                                            "counters"
                                                        ][
                                                            leave
                                                        ] = int(
                                                            mitigation["interfaces"][
                                                                "interface"
                                                            ][idx]["subinterfaces"][
                                                                "subinterface"
                                                            ][
                                                                subidx
                                                            ][
                                                                "state"
                                                            ][
                                                                "counters"
                                                            ][
                                                                leave
                                                            ]
                                                        )
                                                    if (
                                                        mitigation["interfaces"][
                                                            "interface"
                                                        ][idx]["subinterfaces"][
                                                            "subinterface"
                                                        ][
                                                            subidx
                                                        ][
                                                            "state"
                                                        ][
                                                            "counters"
                                                        ][
                                                            leave
                                                        ]
                                                        > 9223372036854775807
                                                    ):
                                                        mitigation["interfaces"][
                                                            "interface"
                                                        ][idx]["subinterfaces"][
                                                            "subinterface"
                                                        ][
                                                            subidx
                                                        ][
                                                            "state"
                                                        ][
                                                            "counters"
                                                        ][
                                                            leave
                                                        ] = (
                                                            mitigation["interfaces"][
                                                                "interface"
                                                            ][idx]["subinterfaces"][
                                                                "subinterface"
                                                            ][
                                                                subidx
                                                            ][
                                                                "state"
                                                            ][
                                                                "counters"
                                                            ][
                                                                leave
                                                            ]
                                                            - 9223372036854775808
                                                        )
                                                else:
                                                    try:
                                                        int(
                                                            mitigation["interfaces"][
                                                                "interface"
                                                            ][idx]["subinterfaces"][
                                                                "subinterface"
                                                            ][
                                                                subidx
                                                            ][
                                                                "state"
                                                            ][
                                                                "counters"
                                                            ][
                                                                leave
                                                            ]
                                                        )
                                                    except ValueError:
                                                        mitigation["interfaces"][
                                                            "interface"
                                                        ][idx]["subinterfaces"][
                                                            "subinterface"
                                                        ][
                                                            subidx
                                                        ][
                                                            "state"
                                                        ][
                                                            "counters"
                                                        ][
                                                            leave
                                                        ] = 0
                                                    else:
                                                        mitigation["interfaces"][
                                                            "interface"
                                                        ][idx]["subinterfaces"][
                                                            "subinterface"
                                                        ][
                                                            subidx
                                                        ][
                                                            "state"
                                                        ][
                                                            "counters"
                                                        ][
                                                            leave
                                                        ] = int(
                                                            mitigation["interfaces"][
                                                                "interface"
                                                            ][idx]["subinterfaces"][
                                                                "subinterface"
                                                            ][
                                                                subidx
                                                            ][
                                                                "state"
                                                            ][
                                                                "counters"
                                                            ][
                                                                leave
                                                            ]
                                                        )


def correct_huaw_enc_path():
    if "encoding_path" in mitigation["collector"]["data"]:
        if "openconfig-interfaces:" in mitigation["collector"]["data"]["encoding_path"]:
            mitigation["collector"]["data"][
                "encoding_path"
            ] = "openconfig-interfaces:interfaces"


if __name__ == "__mod_all_json_data__":
    mod_all_json_data



#
#   pmacct (Promiscuous mode IP Accounting package)
#   pmacct is Copyright (C) 2003-2020 by Paolo Lucente
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
#   pmgrpcd and its components are Copyright (C) 2018-2020 by:
#
#   Matthias Arnold <matthias.arnold@swisscom.com>
#   Raphaël P. Barazzutti <raphael@barazzutti.net>
#   Juan Camilo Cardona <jccardona82@gmail.com>
#   Thomas Graf <thomas.graf@swisscom.com>
#   Paolo Lucente <paolo@pmacct.net>
#
from export_pmgrpcd import Exporter
import os
import ujson as json

class FileExporter(Exporter):
    def __init__(self, output_file):
        self.output_file = output_file

    def process_metric(self, datajsonstring):
        jsondata = json.loads(datajsonstring)
        again_json = json.dumps(jsondata).replace("\n", "")
        # we'll probably remove the loading here
        with open(self.output_file, 'a') as fh:
            fh.write(again_json)
            fh.write("\n")




#
#   pmacct (Promiscuous mode IP Accounting package)
#   pmacct is Copyright (C) 2003-2020 by Paolo Lucente
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
#   pmgrpcd and its components are Copyright (C) 2018-2020 by:
#
#   Matthias Arnold <matthias.arnold@swisscom.com>
#   Raphaël P. Barazzutti <raphael@barazzutti.net>
#   Juan Camilo Cardona <jccardona82@gmail.com>
#   Thomas Graf <thomas.graf@swisscom.com>
#   Paolo Lucente <paolo@pmacct.net>
#
from lib_pmgrpcd import PMGRPCDLOG
import cisco_grpc_dialout_pb2_grpc
from google.protobuf.json_format import MessageToDict
import ujson as json
import lib_pmgrpcd
import time
from export_pmgrpcd import FinalizeTelemetryData
import base64

if lib_pmgrpcd.OPTIONS.cenctype == 'gpbkv':
    import cisco_telemetry_pb2



class gRPCMdtDialoutServicer(cisco_grpc_dialout_pb2_grpc.gRPCMdtDialoutServicer):
    def __init__(self):
        PMGRPCDLOG.info("Cisco: Initializing gRPCMdtDialoutServicer()")

    def MdtDialout(self, msg_iterator, context):
        try:
            grpcPeer = {}
            grpcPeerStr = context.peer()
            (
                grpcPeer["telemetry_proto"],
                grpcPeer["telemetry_node"],
                grpcPeer["telemetry_node_port"],
            ) = grpcPeerStr.split(":")
            grpcPeer["ne_vendor"] = "Cisco"
            PMGRPCDLOG.debug("Cisco MdtDialout Message: %s" % grpcPeer["telemetry_node"])
    
            # cisco_processing(grpcPeer, message, context)
            metadata = dict(context.invocation_metadata())
            grpcPeer["user-agent"] = metadata["user-agent"]
            # Example of grpcPeerStr -> 'ipv4:10.215.133.23:57775'
            grpcPeer["grpc_processing"] = "cisco_grpc_dialout_pb2_grpc"
            grpcPeer["grpc_ulayer"] = "GPB Telemetry"
            jsonTelemetryNode = json.dumps(grpcPeer, indent=2, sort_keys=True)
    
            PMGRPCDLOG.debug("Cisco connection info: %s" % jsonTelemetryNode)
    
            for new_msg in msg_iterator:
                PMGRPCDLOG.debug("Cisco new_msg iteration message")
    
                # filter msgs that do not match the IP option if enabled.
                if lib_pmgrpcd.OPTIONS.ip:
                    if grpcPeer["telemetry_node"] != lib_pmgrpcd.OPTIONS.ip:
                        continue
                    PMGRPCDLOG.debug(
                        "Cisco: ip filter matched with ip %s" % (lib_pmgrpcd.OPTIONS.ip)
                    )
    
                try:
                    cisco_processing(grpcPeer, new_msg)
                except Exception as e:
                    PMGRPCDLOG.debug("Error processing Cisco packet, error is %s", e)
                    continue
        except Exception as e:
            print(type(e))
            print(e.args)
        return
        yield

def process_cisco_kv(new_msg):
    """
    Processes a msg using gpb-kv
    """
    telemetry_msg = cisco_telemetry_pb2.Telemetry()
    telemetry_msg.ParseFromString(new_msg.data)
    #jsonStrTelemetry = MessageToJson(telemetry_msg)
    #grpc_message = json.loads(jsonStrTelemetry)
    grpc_message = MessageToDict(telemetry_msg)
    return grpc_message


def cisco_processing(grpcPeer, new_msg):
    messages = {}
    grpc_message = {}
    encoding_type = None
    PMGRPCDLOG.debug("Cisco: Received GRPC-Data")
    PMGRPCDLOG.debug(new_msg.data)

    # Find the encoding of the packet
    try:
        encoding_type, grpc_message = find_encoding_and_decode(new_msg)
    except Exception as e:
        PMGRPCDLOG.error("Error decoding packet. Error is {}".format(e))


    PMGRPCDLOG.debug("encoding_type is: %s\n" % (encoding_type))

    if (encoding_type == "unknown") or encoding_type is None:
        print("encoding_type is unknown.")


    if (encoding_type == "unknown") or encoding_type is None:
        raise Exception("Encoding type unknown")

    message_header_dict = grpc_message.copy()

    if "data_json" in message_header_dict:
        del message_header_dict["data_json"]

    PMGRPCDLOG.debug("Header:%s", message_header_dict)

    (node_ip) = grpcPeer["telemetry_node"]
    (ne_vendor) = grpcPeer["ne_vendor"]
    epochmillis = int(round(time.time() * 1000))

    if encoding_type == "ciscojson":
        message_header_dict.update({"encoding_type": encoding_type})
        (proto, path) = message_header_dict["encoding_path"].split(":")
        (node_id_str) = message_header_dict["node_id_str"]
        elem = len(grpc_message["data_json"])
        messages = grpc_message["data_json"]
    elif encoding_type == "ciscogrpckv":
        message_header_dict.update({"encoding_type": encoding_type})
        message_header_dict["encoding_path"] = message_header_dict.pop("encodingPath")
        message_header_dict["node_id_str"] = message_header_dict.pop("nodeIdStr")
        message_header_dict["msg_timestamp"] = message_header_dict.pop("msgTimestamp")
        message_header_dict["subscription_id_str"] = message_header_dict.pop(
            "subscriptionIdStr"
        )

        (proto, path) = message_header_dict["encoding_path"].split(":")
        (node_id_str) = message_header_dict["node_id_str"]
        if "dataGpbkv" in grpc_message:
            elem = len(grpc_message["dataGpbkv"])
            messages = grpc_message["dataGpbkv"]
        else:
            elem = 0
            messages = {}

    PMGRPCDLOG.info(
        "EPOCH=%-10s NIP=%-15s NID=%-20s VEN=%-7s PT=%-22s ET=%-12s ELEM=%s",
        epochmillis,
        node_ip,
        node_id_str,
        ne_vendor,
        proto,
        encoding_type,
        elem,
    )

    # A single telemetry packet can contain multiple msgs (each having their own key/values).
    # here we are processing them one by one.

    for listelem in messages:
        # Copy the necessary metadata to the packet.
        PMGRPCDLOG.debug("LISTELEM: %s", listelem)

        message_dict = {}
        message_dict.update({"collector": {"grpc": {}}})
        message_dict["collector"]["grpc"].update(
            {"grpcPeer": grpcPeer["telemetry_node"]}
        )
        message_dict["collector"]["grpc"].update({"ne_vendor": grpcPeer["ne_vendor"]})
        message_dict["collector"].update({"data": message_header_dict})

        if encoding_type == "ciscojson":
            PMGRPCDLOG.debug("TEST: %s | %s", path, listelem["content"])
            message_dict.update({path: listelem["content"]})
        elif encoding_type == "ciscogrpckv":
            PMGRPCDLOG.debug("TEST: %s | %s", path, listelem["fields"])
            message_dict.update({path: listelem["fields"]})

        # allkeys = parse_dict(listelem, ret='', level=0)
        # PMGRPCDLOG.info("Cisco: %s: %s" % (proto, allkeys))


        # dump the raw data
        if lib_pmgrpcd.OPTIONS.rawdatadumpfile:
            PMGRPCDLOG.debug("Write rawdatadumpfile: %s" % (lib_pmgrpcd.OPTIONS.rawdatadumpfile))
            with open(lib_pmgrpcd.OPTIONS.rawdatadumpfile, "a") as rawdatadumpfile:
                 rawdatadumpfile.write(json.dumps(message_dict, indent=2, sort_keys=True))
                 rawdatadumpfile.write("\n")

        try:
            returned = FinalizeTelemetryData(message_dict)
        except Exception as e:
            PMGRPCDLOG.error("Error finalizing  message: %s", e)




def find_encoding_and_decode(new_msg):
    encoding_type = None
    grpc_message = {}

    # TODO. If options force one type, only try that one.
    # Maybe it is json
    if lib_pmgrpcd.OPTIONS.cenctype == 'json':
        PMGRPCDLOG.debug("Try to parse json")
        try:
            grpc_message = json.loads(new_msg.data)
            encoding_type = "ciscojson"
        except Exception as e:
            PMGRPCDLOG.debug(
                "ERROR: Direct json parsing of grpc_message failed with message:\n%s\n", e
            )
        else:
            return encoding_type, grpc_message

    elif lib_pmgrpcd.OPTIONS.cenctype == 'gpbkv':
        PMGRPCDLOG.debug("Try to unmarshall KV")
        if encoding_type is None:
            try:
                grpc_message = process_cisco_kv(new_msg)
                encoding_type = "ciscogrpckv"
            except Exception as e:
                PMGRPCDLOG.debug(
                    "ERROR: Parsing of json after unmarshall KV failed with message:\n%s\n",
                    e,
                )
            else:
                return encoding_type, grpc_message
    
    elif lib_pmgrpcd.OPTIONS.cenctype == 'gpbcomp':
        PMGRPCDLOG.debug("Try to unmarshall compact mode")
        PMGRPCDLOG.debug("TODO")

    encoding_type = "unknown"
    return encoding_type, grpc_message



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
import grpc

import huawei_grpc_dialout_pb2 as huawei__grpc__dialout__pb2


class gRPCDataserviceStub(object):
  # missing associated documentation comment in .proto file
  pass

  def __init__(self, channel):
    """Constructor.

    Args:
      channel: A grpc.Channel.
    """
    self.dataPublish = channel.stream_stream(
        '/huawei_dialout.gRPCDataservice/dataPublish',
        request_serializer=huawei__grpc__dialout__pb2.serviceArgs.SerializeToString,
        response_deserializer=huawei__grpc__dialout__pb2.serviceArgs.FromString,
        )


class gRPCDataserviceServicer(object):
  # missing associated documentation comment in .proto file
  pass

  def dataPublish(self, request_iterator, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')


def add_gRPCDataserviceServicer_to_server(servicer, server):
  rpc_method_handlers = {
      'dataPublish': grpc.stream_stream_rpc_method_handler(
          servicer.dataPublish,
          request_deserializer=huawei__grpc__dialout__pb2.serviceArgs.FromString,
          response_serializer=huawei__grpc__dialout__pb2.serviceArgs.SerializeToString,
      ),
  }
  generic_handler = grpc.method_handlers_generic_handler(
      'huawei_dialout.gRPCDataservice', rpc_method_handlers)
  server.add_generic_rpc_handlers((generic_handler,))



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: huawei-ifm.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='huawei-ifm.proto',
  package='huawei_ifm',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=_b('\n\x10huawei-ifm.proto\x12\nhuawei_ifm\"\xc1\x08\n\x03Ifm\x12.\n\ninterfaces\x18\x05 \x01(\x0b\x32\x1a.huawei_ifm.Ifm.Interfaces\x1a\x89\x08\n\nInterfaces\x12\x37\n\tinterface\x18\x01 \x03(\x0b\x32$.huawei_ifm.Ifm.Interfaces.Interface\x1a\xc1\x07\n\tInterface\x12I\n\rifAdminStatus\x18\x05 \x01(\x0e\x32\x32.huawei_ifm.Ifm.Interfaces.Interface.IfAdminStatus\x12I\n\rifClearedStat\x18\t \x01(\x0b\x32\x32.huawei_ifm.Ifm.Interfaces.Interface.IfClearedStat\x12I\n\rifDynamicInfo\x18\r \x01(\x0b\x32\x32.huawei_ifm.Ifm.Interfaces.Interface.IfDynamicInfo\x12\x0f\n\x07ifIndex\x18\x10 \x01(\r\x12\x0e\n\x06ifName\x18\x19 \x01(\t\x12G\n\x0cifStatistics\x18# \x01(\x0b\x32\x31.huawei_ifm.Ifm.Interfaces.Interface.IfStatistics\x1a\x35\n\rIfClearedStat\x12\x10\n\x08sendByte\x18\x1d \x01(\x04\x12\x12\n\nsendPacket\x18$ \x01(\x04\x1a\xa2\x01\n\rIfDynamicInfo\x12U\n\x0cifOperStatus\x18\x05 \x01(\x0e\x32?.huawei_ifm.Ifm.Interfaces.Interface.IfDynamicInfo.IfOperStatus\":\n\x0cIfOperStatus\x12\x15\n\x11IfOperStatus_DOWN\x10\x00\x12\x13\n\x0fIfOperStatus_UP\x10\x01\x1a\xcd\x02\n\x0cIfStatistics\x12\x16\n\x0ercvBroadPacket\x18\x02 \x01(\x04\x12\x15\n\rrcvDropPacket\x18\x03 \x01(\x04\x12\x16\n\x0ercvErrorPacket\x18\x04 \x01(\x04\x12\x15\n\rrcvMutiPacket\x18\x05 \x01(\x04\x12\x14\n\x0crcvUniPacket\x18\x06 \x01(\x04\x12\x13\n\x0breceiveByte\x18\x07 \x01(\x04\x12\x15\n\rreceivePacket\x18\x08 \x01(\x04\x12\x17\n\x0fsendBroadPacket\x18\t \x01(\x04\x12\x10\n\x08sendByte\x18\n \x01(\x04\x12\x16\n\x0esendDropPacket\x18\x0b \x01(\x04\x12\x17\n\x0fsendErrorPacket\x18\x0c \x01(\x04\x12\x16\n\x0esendMutiPacket\x18\r \x01(\x04\x12\x12\n\nsendPacket\x18\x0e \x01(\x04\x12\x15\n\rsendUniPacket\x18\x0f \x01(\x04\"=\n\rIfAdminStatus\x12\x16\n\x12IfAdminStatus_DOWN\x10\x00\x12\x14\n\x10IfAdminStatus_UP\x10\x01\x62\x06proto3')
)



_IFM_INTERFACES_INTERFACE_IFDYNAMICINFO_IFOPERSTATUS = _descriptor.EnumDescriptor(
  name='IfOperStatus',
  full_name='huawei_ifm.Ifm.Interfaces.Interface.IfDynamicInfo.IfOperStatus',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='IfOperStatus_DOWN', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='IfOperStatus_UP', index=1, number=1,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=665,
  serialized_end=723,
)
_sym_db.RegisterEnumDescriptor(_IFM_INTERFACES_INTERFACE_IFDYNAMICINFO_IFOPERSTATUS)

_IFM_INTERFACES_INTERFACE_IFADMINSTATUS = _descriptor.EnumDescriptor(
  name='IfAdminStatus',
  full_name='huawei_ifm.Ifm.Interfaces.Interface.IfAdminStatus',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='IfAdminStatus_DOWN', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='IfAdminStatus_UP', index=1, number=1,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1061,
  serialized_end=1122,
)
_sym_db.RegisterEnumDescriptor(_IFM_INTERFACES_INTERFACE_IFADMINSTATUS)


_IFM_INTERFACES_INTERFACE_IFCLEAREDSTAT = _descriptor.Descriptor(
  name='IfClearedStat',
  full_name='huawei_ifm.Ifm.Interfaces.Interface.IfClearedStat',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='sendByte', full_name='huawei_ifm.Ifm.Interfaces.Interface.IfClearedStat.sendByte', index=0,
      number=29, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sendPacket', full_name='huawei_ifm.Ifm.Interfaces.Interface.IfClearedStat.sendPacket', index=1,
      number=36, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=505,
  serialized_end=558,
)

_IFM_INTERFACES_INTERFACE_IFDYNAMICINFO = _descriptor.Descriptor(
  name='IfDynamicInfo',
  full_name='huawei_ifm.Ifm.Interfaces.Interface.IfDynamicInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='ifOperStatus', full_name='huawei_ifm.Ifm.Interfaces.Interface.IfDynamicInfo.ifOperStatus', index=0,
      number=5, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _IFM_INTERFACES_INTERFACE_IFDYNAMICINFO_IFOPERSTATUS,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=561,
  serialized_end=723,
)

_IFM_INTERFACES_INTERFACE_IFSTATISTICS = _descriptor.Descriptor(
  name='IfStatistics',
  full_name='huawei_ifm.Ifm.Interfaces.Interface.IfStatistics',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='rcvBroadPacket', full_name='huawei_ifm.Ifm.Interfaces.Interface.IfStatistics.rcvBroadPacket', index=0,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='rcvDropPacket', full_name='huawei_ifm.Ifm.Interfaces.Interface.IfStatistics.rcvDropPacket', index=1,
      number=3, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='rcvErrorPacket', full_name='huawei_ifm.Ifm.Interfaces.Interface.IfStatistics.rcvErrorPacket', index=2,
      number=4, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='rcvMutiPacket', full_name='huawei_ifm.Ifm.Interfaces.Interface.IfStatistics.rcvMutiPacket', index=3,
      number=5, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='rcvUniPacket', full_name='huawei_ifm.Ifm.Interfaces.Interface.IfStatistics.rcvUniPacket', index=4,
      number=6, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='receiveByte', full_name='huawei_ifm.Ifm.Interfaces.Interface.IfStatistics.receiveByte', index=5,
      number=7, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='receivePacket', full_name='huawei_ifm.Ifm.Interfaces.Interface.IfStatistics.receivePacket', index=6,
      number=8, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sendBroadPacket', full_name='huawei_ifm.Ifm.Interfaces.Interface.IfStatistics.sendBroadPacket', index=7,
      number=9, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sendByte', full_name='huawei_ifm.Ifm.Interfaces.Interface.IfStatistics.sendByte', index=8,
      number=10, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sendDropPacket', full_name='huawei_ifm.Ifm.Interfaces.Interface.IfStatistics.sendDropPacket', index=9,
      number=11, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sendErrorPacket', full_name='huawei_ifm.Ifm.Interfaces.Interface.IfStatistics.sendErrorPacket', index=10,
      number=12, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sendMutiPacket', full_name='huawei_ifm.Ifm.Interfaces.Interface.IfStatistics.sendMutiPacket', index=11,
      number=13, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sendPacket', full_name='huawei_ifm.Ifm.Interfaces.Interface.IfStatistics.sendPacket', index=12,
      number=14, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sendUniPacket', full_name='huawei_ifm.Ifm.Interfaces.Interface.IfStatistics.sendUniPacket', index=13,
      number=15, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=726,
  serialized_end=1059,
)

_IFM_INTERFACES_INTERFACE = _descriptor.Descriptor(
  name='Interface',
  full_name='huawei_ifm.Ifm.Interfaces.Interface',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='ifAdminStatus', full_name='huawei_ifm.Ifm.Interfaces.Interface.ifAdminStatus', index=0,
      number=5, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='ifClearedStat', full_name='huawei_ifm.Ifm.Interfaces.Interface.ifClearedStat', index=1,
      number=9, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='ifDynamicInfo', full_name='huawei_ifm.Ifm.Interfaces.Interface.ifDynamicInfo', index=2,
      number=13, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='ifIndex', full_name='huawei_ifm.Ifm.Interfaces.Interface.ifIndex', index=3,
      number=16, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='ifName', full_name='huawei_ifm.Ifm.Interfaces.Interface.ifName', index=4,
      number=25, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='ifStatistics', full_name='huawei_ifm.Ifm.Interfaces.Interface.ifStatistics', index=5,
      number=35, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_IFM_INTERFACES_INTERFACE_IFCLEAREDSTAT, _IFM_INTERFACES_INTERFACE_IFDYNAMICINFO, _IFM_INTERFACES_INTERFACE_IFSTATISTICS, ],
  enum_types=[
    _IFM_INTERFACES_INTERFACE_IFADMINSTATUS,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=161,
  serialized_end=1122,
)

_IFM_INTERFACES = _descriptor.Descriptor(
  name='Interfaces',
  full_name='huawei_ifm.Ifm.Interfaces',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='interface', full_name='huawei_ifm.Ifm.Interfaces.interface', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_IFM_INTERFACES_INTERFACE, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=89,
  serialized_end=1122,
)

_IFM = _descriptor.Descriptor(
  name='Ifm',
  full_name='huawei_ifm.Ifm',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='interfaces', full_name='huawei_ifm.Ifm.interfaces', index=0,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_IFM_INTERFACES, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=33,
  serialized_end=1122,
)

_IFM_INTERFACES_INTERFACE_IFCLEAREDSTAT.containing_type = _IFM_INTERFACES_INTERFACE
_IFM_INTERFACES_INTERFACE_IFDYNAMICINFO.fields_by_name['ifOperStatus'].enum_type = _IFM_INTERFACES_INTERFACE_IFDYNAMICINFO_IFOPERSTATUS
_IFM_INTERFACES_INTERFACE_IFDYNAMICINFO.containing_type = _IFM_INTERFACES_INTERFACE
_IFM_INTERFACES_INTERFACE_IFDYNAMICINFO_IFOPERSTATUS.containing_type = _IFM_INTERFACES_INTERFACE_IFDYNAMICINFO
_IFM_INTERFACES_INTERFACE_IFSTATISTICS.containing_type = _IFM_INTERFACES_INTERFACE
_IFM_INTERFACES_INTERFACE.fields_by_name['ifAdminStatus'].enum_type = _IFM_INTERFACES_INTERFACE_IFADMINSTATUS
_IFM_INTERFACES_INTERFACE.fields_by_name['ifClearedStat'].message_type = _IFM_INTERFACES_INTERFACE_IFCLEAREDSTAT
_IFM_INTERFACES_INTERFACE.fields_by_name['ifDynamicInfo'].message_type = _IFM_INTERFACES_INTERFACE_IFDYNAMICINFO
_IFM_INTERFACES_INTERFACE.fields_by_name['ifStatistics'].message_type = _IFM_INTERFACES_INTERFACE_IFSTATISTICS
_IFM_INTERFACES_INTERFACE.containing_type = _IFM_INTERFACES
_IFM_INTERFACES_INTERFACE_IFADMINSTATUS.containing_type = _IFM_INTERFACES_INTERFACE
_IFM_INTERFACES.fields_by_name['interface'].message_type = _IFM_INTERFACES_INTERFACE
_IFM_INTERFACES.containing_type = _IFM
_IFM.fields_by_name['interfaces'].message_type = _IFM_INTERFACES
DESCRIPTOR.message_types_by_name['Ifm'] = _IFM
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Ifm = _reflection.GeneratedProtocolMessageType('Ifm', (_message.Message,), {

  'Interfaces' : _reflection.GeneratedProtocolMessageType('Interfaces', (_message.Message,), {

    'Interface' : _reflection.GeneratedProtocolMessageType('Interface', (_message.Message,), {

      'IfClearedStat' : _reflection.GeneratedProtocolMessageType('IfClearedStat', (_message.Message,), {
        'DESCRIPTOR' : _IFM_INTERFACES_INTERFACE_IFCLEAREDSTAT,
        '__module__' : 'huawei_ifm_pb2'
        # @@protoc_insertion_point(class_scope:huawei_ifm.Ifm.Interfaces.Interface.IfClearedStat)
        })
      ,

      'IfDynamicInfo' : _reflection.GeneratedProtocolMessageType('IfDynamicInfo', (_message.Message,), {
        'DESCRIPTOR' : _IFM_INTERFACES_INTERFACE_IFDYNAMICINFO,
        '__module__' : 'huawei_ifm_pb2'
        # @@protoc_insertion_point(class_scope:huawei_ifm.Ifm.Interfaces.Interface.IfDynamicInfo)
        })
      ,

      'IfStatistics' : _reflection.GeneratedProtocolMessageType('IfStatistics', (_message.Message,), {
        'DESCRIPTOR' : _IFM_INTERFACES_INTERFACE_IFSTATISTICS,
        '__module__' : 'huawei_ifm_pb2'
        # @@protoc_insertion_point(class_scope:huawei_ifm.Ifm.Interfaces.Interface.IfStatistics)
        })
      ,
      'DESCRIPTOR' : _IFM_INTERFACES_INTERFACE,
      '__module__' : 'huawei_ifm_pb2'
      # @@protoc_insertion_point(class_scope:huawei_ifm.Ifm.Interfaces.Interface)
      })
    ,
    'DESCRIPTOR' : _IFM_INTERFACES,
    '__module__' : 'huawei_ifm_pb2'
    # @@protoc_insertion_point(class_scope:huawei_ifm.Ifm.Interfaces)
    })
  ,
  'DESCRIPTOR' : _IFM,
  '__module__' : 'huawei_ifm_pb2'
  # @@protoc_insertion_point(class_scope:huawei_ifm.Ifm)
  })
_sym_db.RegisterMessage(Ifm)
_sym_db.RegisterMessage(Ifm.Interfaces)
_sym_db.RegisterMessage(Ifm.Interfaces.Interface)
_sym_db.RegisterMessage(Ifm.Interfaces.Interface.IfClearedStat)
_sym_db.RegisterMessage(Ifm.Interfaces.Interface.IfDynamicInfo)
_sym_db.RegisterMessage(Ifm.Interfaces.Interface.IfStatistics)


# @@protoc_insertion_point(module_scope)



#
#   pmacct (Promiscuous mode IP Accounting package)
#   pmacct is Copyright (C) 2003-2020 by Paolo Lucente
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
#   pmgrpcd and its components are Copyright (C) 2018-2020 by:
#
#   Matthias Arnold <matthias.arnold@swisscom.com>
#   Raphaël P. Barazzutti <raphael@barazzutti.net>
#   Juan Camilo Cardona <jccardona82@gmail.com>
#   Thomas Graf <thomas.graf@swisscom.com>
#   Paolo Lucente <paolo@pmacct.net>
#
from confluent_kafka import Consumer, KafkaError
import sys
from optparse import OptionParser

DEFAULT_TOPIC = "daisy.test.device-avro-raw"
DEFAULT_SERVER = "kafka.sbd.corproot.net:9093"
parser = OptionParser()
parser.add_option(
    "-t",
    "--topic",,
    default=str(DEFAULT_TOPIC),
    dest="topic",
    help="Topic to listen",
)
parser.add_option(
    "-s",
    "--servers",,
    default=str(DEFAULT_SERVER),
    dest="servers",
    help="Kafka servers",
)

(options, _) = parser.parse_args()

c = Consumer({
    'bootstrap.servers': options.servers,
    'group.id': 'mygroup',
    'auto.offset.reset': 'earliest'
})

c.subscribe([options.topic])
#c.subscribe(['Cisco-IOS-XR-qos-ma-oper.qos.nodes.node.policy-map.interface-table.interface.member-interfaces.member-interface.output.service-policy-names.service-policy-instance.statistics'])

while True:
    msg = c.poll(1.0)

    if msg is None:
        continue
    if msg.error():
        print("Consumer error: {}".format(msg.error()))
        continue

    print('Received message: {}'.format(msg.value().decode('utf-8')))

c.close()



#!/usr/bin/env python
#
#   pmacct (Promiscuous mode IP Accounting package)
#   pmacct is Copyright (C) 2003-2018 by Paolo Lucente
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#

from concurrent import futures
import sys, getopt
import ujson as json
import zmq
import time

# gRPC and Protobuf imports
import grpc
from google.protobuf.json_format import MessageToJson
from google.protobuf.json_format import MessageToDict
import huawei_grpc_dialout_pb2
import huawei_grpc_dialout_pb2_grpc
import huawei_telemetry_pb2
import huawei_ifm_pb2
import huawei_devm_pb2
import openconfig_interfaces_pb2

_ONE_DAY_IN_SECONDS = 60 * 60 * 24

debug = False
zmqContext = None
zmqSock = None

class gRPCDataserviceServicer(huawei_grpc_dialout_pb2_grpc.gRPCDataserviceServicer):
	def __init__(self):
		print "Initializing gRPCDataserviceServicer()"

	def dataPublish(self, message, context):
		grpcPeerStr = context.peer() 

		grpcPeer = {}
		(grpcPeerProto, grpcPeer['telemetry_node'], grpcPeer['telemetry_node_port']) = grpcPeerStr.split(":")
		grpcPeer['telemetry_node_vendor'] = "Huawei"
		jsonTelemetryNode = json.dumps(grpcPeer)

		for new_msg in message:
			telemetry_msg = huawei_telemetry_pb2.Telemetry()
			telemetry_msg.ParseFromString(new_msg.data)
			jsonStrTelemetry = MessageToJson(telemetry_msg)
			dictTelemetry = json.loads(jsonStrTelemetry)

			(proto, path) = telemetry_msg.sensor_path.split(":")

			if (proto == "huawei-ifm"):
				for new_row in telemetry_msg.data_gpb.row:
					dictHuaweiIfm = {}

					huaweiIfm_msg = huawei_ifm_pb2.Ifm()
					huaweiIfm_msg.ParseFromString(new_row.content)
					dictHuaweiIfmContent = MessageToDict(huaweiIfm_msg,
										including_default_value_fields = True,
										preserving_proto_field_name = True,
										use_integers_for_enums = True)

					dictHuaweiIfm['row'] = {}
					dictHuaweiIfm['row']['content'] = dictHuaweiIfmContent
					dictHuaweiIfm['row']['timestamp'] = new_row.timestamp

					dictTelemetry['dataGpb'] = dictHuaweiIfm
					sendJsonTelemetryData(jsonTelemetryNode, dictTelemetry)

			elif (proto == "huawei-devm"):
				for new_row in telemetry_msg.data_gpb.row:
					dictHuaweiDevm = {}

					huaweiDevm_msg = huawei_devm_pb2.Devm()
					huaweiDevm_msg.ParseFromString(new_row.content)
					dictHuaweiDevmContent = MessageToDict(huaweiDevm_msg,
										including_default_value_fields = True,
										preserving_proto_field_name = True,
										use_integers_for_enums = True)

					dictHuaweiDevm['row'] = {}
					dictHuaweiDevm['row']['content'] = dictHuaweiDevmContent 
					dictHuaweiDevm['row']['timestamp'] = new_row.timestamp

					dictTelemetry['dataGpb'] = dictHuaweiDevm
					sendJsonTelemetryData(jsonTelemetryNode, dictTelemetry)

			elif (proto == "openconfig-interfaces"):
				for new_row in telemetry_msg.data_gpb.row:
					dictOcInterfaces = {}

					ocInterfaces_msg = openconfig_interfaces_pb2.Interfaces()
					ocInterfaces_msg.ParseFromString(new_row.content)
					dictOcInterfacesContent = MessageToDict(ocInterfaces_msg,
										including_default_value_fields = True,
										preserving_proto_field_name = True,
										use_integers_for_enums = True)

					dictOcInterfaces['row'] = {}
					dictOcInterfaces['row']['content'] = dictOcInterfacesContent
					dictOcInterfaces['row']['timestamp'] = new_row.timestamp

					dictTelemetry['dataGpb'] = dictOcInterfaces
					sendJsonTelemetryData(jsonTelemetryNode, dictTelemetry)

			else:
				sendJsonTelemetryData(jsonTelemetryNode, dictTelemetry)

def sendJsonTelemetryData(jsonTelemetryNode, dictTelemetryData):
	global debug
	global zmqSock

	jsonTelemetryData = json.dumps(dictTelemetryData) 

	if debug:
		print "dataPublish(): +++"
		print "dataPublish(): " + jsonTelemetryNode
		print "dataPublish(): " + jsonTelemetryData
		print "dataPublish(): ---"

	if not zmqSock.closed:
		try:
			zmqSock.send("%s" % jsonTelemetryNode, flags=zmq.SNDMORE)
			zmqSock.send("%s" % jsonTelemetryData)
		except ZMQError:
			pass

def usage(tool):
	print ""
	print "Usage: %s [Args]" % tool
	print ""

	print "Optional Args:"
	print "  -h, --help".ljust(25) + "Print this help"
	print "  -d, --debug".ljust(25) + "Enable debug [default: no]"
	print "  -g, --grpc-port".ljust(25) + "Set the port for input gRPC sessions [default: '10000']"
	print "  -Z, --zmq-host".ljust(25) + "Set the ZeroMQ host for output [default: '127.0.0.1']"
	print "  -z, --zmq-port".ljust(25) + "Set the ZeroMQ port for output [default: '50000']"

def serve():
	global debug
	global zmqContext
	global zmqSock

	try:
		opts, args = getopt.getopt(sys.argv[1:], "hdg:Z:z:", ["help", "debug", "grpc-port=", "zmq-host=", "zmq-port="])
	except getopt.GetoptError as err:
		print str(err)
		usage(sys.argv[0])
		sys.exit(1)

	# Defaults
	debug = False
	grpc_port = 10001
	zmq_host = "127.0.0.1"
	zmq_port = 50001

	for o, a in opts:
		if o in ("-h", "--help"):
			usage(sys.argv[0])
			sys.exit(0)
		elif o in ("-d", "--debug"):
			debug = True
		elif o in ("-g", "--grpc-port"):
			grpc_port = a
		elif o in ("-Z", "--zmq-host"):
			zmq_host = a
		elif o in ("-z", "--zmq-port"):
			zmq_port = a
		else:
			assert False, "unhandled option"

	zmqBindStr = "tcp://" + zmq_host + ":" + str(zmq_port)
	gRPCBindStr = "[::]:" + str(grpc_port)

	zmqContext = zmq.Context()
	zmqSock = zmqContext.socket(zmq.PUSH)
	zmqSock.bind(zmqBindStr)

	gRPCserver = grpc.server(futures.ThreadPoolExecutor(max_workers=10))
	huawei_grpc_dialout_pb2_grpc.add_gRPCDataserviceServicer_to_server(gRPCDataserviceServicer(), gRPCserver)
	gRPCserver.add_insecure_port(gRPCBindStr)
	gRPCserver.start()

	try:
		while True:
			time.sleep(_ONE_DAY_IN_SECONDS)
	except KeyboardInterrupt:
		gRPCserver.stop(0)

if __name__ == '__main__':
	serve()



#
#   pmacct (Promiscuous mode IP Accounting package)
#   pmacct is Copyright (C) 2003-2020 by Paolo Lucente
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
#   pmgrpcd and its components are Copyright (C) 2018-2020 by:
#
#   Matthias Arnold <matthias.arnold@swisscom.com>
#   Raphaël P. Barazzutti <raphael@barazzutti.net>
#   Juan Camilo Cardona <jccardona82@gmail.com>
#   Thomas Graf <thomas.graf@swisscom.com>
#   Paolo Lucente <paolo@pmacct.net>
#
from huawei_generator  import HuaweiDialOutClient
from  huawei_grpc_dialout_pb2  import serviceArgs
from utils import generate_content_from_raw
from optparse import OptionParser

DEFAULT_FILE = "huawei_dump"
DEFAULT_CONNECTION = "127.0.0.1:6000"

parser = OptionParser()
parser.add_option(
    "-f",
    "--file",
    default=str(DEFAULT_FILE),
    dest="file",
    help="File with raw data",
)
parser.add_option(
    "-c",
    "--connection",
    default=str(DEFAULT_CONNECTION),
    help="IP (socket address) of the collector",
)

(options, _) = parser.parse_args()


huawei_client = HuaweiDialOutClient(options.connection)
def generate_data(data_generator):
    for data in data_generator:
        yield serviceArgs(ReqId=1, data=data)

huawei_client.send_data(generate_data(generate_content_from_raw(options.file)))

# check status
while not huawei_client.rcv.done():
    continue
try:
    result = huawei_client.rcv.result()
except Exception as e:
    print("Generation failed with error ", e)








#
#   pmacct (Promiscuous mode IP Accounting package)
#   pmacct is Copyright (C) 2003-2020 by Paolo Lucente
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
#   pmgrpcd and its components are Copyright (C) 2018-2020 by:
#
#   Matthias Arnold <matthias.arnold@swisscom.com>
#   Raphaël P. Barazzutti <raphael@barazzutti.net>
#   Juan Camilo Cardona <jccardona82@gmail.com>
#   Thomas Graf <thomas.graf@swisscom.com>
#   Paolo Lucente <paolo@pmacct.net>
#
import multiprocessing
import threading

from confluent_kafka.avro.cached_schema_registry_client import (
    CachedSchemaRegistryClient,
)
from confluent_kafka import avro
from confluent_kafka.avro import AvroProducer
import lib_pmgrpcd
import ujson as json
from export_pmgrpcd import Exporter
from lib_pmgrpcd import PMGRPCDLOG
import logging
import logging.handlers
import signal
import os


class WorkerProcess(multiprocessing.Process):
    def __init__(self, queue, state_builder, transform_function):
        super().__init__()
        self.__queue = queue
        self.__state_builder = state_builder
        self.__transformFunction = transform_function

    def run(self):
        state = self.__state_builder()
        while True:
            submission = self.__queue.get(block=True)
            if submission is not None:
                ret = self.__transformFunction(state, submission.job)
                if submission.callback is not None:
                    submission.callback(ret)
            else:
                return


class WorkerSwarm:
    def enqueue(self, job, callback=None):
        self.__queue.put(WorkerTask(job, callback))

    def __init__(self, number_of_workers, state_builder, transform_function):
        self.__queue = multiprocessing.Queue()
        self.__processes = []
        for i in range(0, number_of_workers):
            t = WorkerProcess(self.__queue, state_builder, transform_function)
            self.__processes.append(t)

    def start(self):
        for process in self.__processes:
            process.start()

    def wait(self):
        for process in self.__processes:
            process.join()

    def stop(self):
        self.__queue.empty()
        for process in self.__processes:
            self.__queue.put(None, block=True)


class WorkerTask:
    def __init__(self, job, callback):
        super().__init__()
        self.job = job
        self.callback = callback


class KafkaAvroExporterContext:
    def __init__(self):
        global logQueue
        self.avscmap = {}
        self.jsonmap = {}
        # https://docs.python.org/3/howto/logging-cookbook.html#logging-to-a-single-file-from-multiple-processes
        h = logging.handlers.QueueHandler(logQueue)  # Just the one handler needed
        root = logging.getLogger()
        root.addHandler(h)
        # send all messages, for demo; no other level or filter logic applied.
        root.setLevel(logging.DEBUG)
        self.__logger = logging.getLogger("KAFKA-AVRO-WORKER")

    def process_metric(self, datajsonstring):
        jsondata = json.loads(datajsonstring)
        self.__logger.debug("In process_metric")

        if "grpcPeer" in jsondata["collector"]["grpc"]:
            grpcPeer = jsondata["collector"]["grpc"]["grpcPeer"]

            if "collection_timestamp" in jsondata["collector"]["data"]:
                collection_timestamp = jsondata["collector"]["data"][
                    "collection_timestamp"
                ]
            else:
                collection_timestamp = -1

            if "encoding_path" in jsondata["collector"]["data"]:
                encoding_path = jsondata["collector"]["data"]["encoding_path"]

                # print("IDENIFIER: %s - %s" % (grpcPeer, encoding_path))
                self.__logger.debug("Found encoding_path: %s" % encoding_path)
                avscid = self.getavroschemaid(grpcPeer, encoding_path)
                # print("AVSCID: %s" % (avscid))
                if avscid is not None:
                    self.__logger.debug(
                        "GETAVROSCHEMAID: grpcPeer=%s | encoding_path=%s | avroschemaid=%s"
                        % (grpcPeer, encoding_path, avscid)
                    )
                    avsc = self.getavroschema(avscid)
                    avroinstance = self.getavro_schid_instance(avscid)
                    self.__logger.debug("avroinstance is: %s" % (avroinstance))

                    if "name" in avsc:
                        self.__logger.info(
                            "SERIALIZE: epoch=%-10s | gP=%-13s | ep=%s | avscid=%s(%s)"
                            % (
                                collection_timestamp,
                                grpcPeer,
                                encoding_path,
                                avscid,
                                avsc["name"],
                            )
                        )
                        topic = lib_pmgrpcd.OPTIONS.topic
                        try:
                            # serialize(json.dumps(avsc), jsondata, topic, avscid, avroinstance)
                            self.serialize(
                                jsondata,
                                lib_pmgrpcd.OPTIONS.topic,
                                avscid,
                                avroinstance,
                            )
                        except Exception as e:
                            if "msg_timestamp" in jsondata["collector"]["data"]:
                                msg_timestamp = jsondata["collector"]["data"][
                                    "msg_timestamp"
                                ]
                            self.__logger.info(
                                "ERROR: serialize exeption on collection_timestamp=%s topic=%s avscid=%s grpcPeer=%s encoding_path=%s msg_timestamp=%s avroschemaname:%s"
                                % (
                                    collection_timestamp,
                                    topic,
                                    avscid,
                                    grpcPeer,
                                    encoding_path,
                                    msg_timestamp,
                                    avsc["name"],
                                )
                            )
                            self.__logger.info("ERROR: %s" % (e))
                            pass
            else:
                self.__logger.info("%s -> encoding_path is missing" % grpcPeer)
        else:
            self.__logger.info("grpcPeer is missing" % jsondata)

    def getavroschemaid(self, grpcPeer, encoding_path):
        # global self.jsonmap
        self.__logger.debug(
            "In getavroschemaid with encoding_path: %s and grpcpeer: %s"
            % (encoding_path, grpcPeer)
        )
        avroid = None
        if type(self.jsonmap) != dict:
            self.__logger.debug("jsonmap is not a dict")
            self.loadavscidmapfile()
        if not self.jsonmap:
            self.__logger.debug("jsonmap is empty")
            self.loadavscidmapfile()
        if grpcPeer in self.jsonmap:
            if encoding_path in self.jsonmap[grpcPeer]:
                avroid = self.jsonmap[grpcPeer][encoding_path]
                self.__logger.debug("avroid is found: %s" % avroid)
            else:
                self.__logger.debug(
                    "avroid not found because of not maching/existing encoding_path (%s) within the mapping and grpcpeer (%s)"
                    % (encoding_path, grpcPeer)
                )
                pass
        else:
            self.__logger.debug(
                "avroid not found because of not maching/existing grpcPeer (%s) within the mapping"
                % grpcPeer
            )
        return avroid

    def loadavscidmapfile(self):
        # global self.jsonmap
        self.__logger.info(
            "loading of the schemaidmappingfile (%s) to the cache self.jsonmap"
            % (lib_pmgrpcd.OPTIONS.avscmapfile)
        )
        with open(lib_pmgrpcd.OPTIONS.avscmapfile, "r") as avscmapfile:
            self.jsonmap = json.load(avscmapfile)

        # mapfile and self.jsonmap
        # -------------------
        # {
        #  "138.187.58.1": {
        #    "openconfig-interfaces:interfaces": 288
        #  },
        #  "10.0.0.2": {
        #    "openconfig-interfaces:interfaces": 288
        #  }
        # }

    def getavro_schid_instance(self, avscid):
        # global  self.avscmap
        self.__logger.debug("In getavro_schid_instance with avscid: %s" % avscid)
        avroinstance = None
        if avscid in self.avscmap:
            if "avroinstance" in self.avscmap[avscid]:
                self.__logger.debug("avroinstance found in dict  self.avscmap")
                avroinstance = self.avscmap[avscid]["avroinstance"]
            else:
                self.__logger.debug("avroinstance not found in dict  self.avscmap")
                self.create_avro_schid_instance(avscid)
                if "avroinstance" in self.avscmap[avscid]:
                    self.__logger.debug(
                        "avroinstance found in dict  self.avscmap after creating with create_avro_schid_instance"
                    )
                    avroinstance = self.avscmap[avscid]["avroinstance"]
        else:
            self.__logger.debug("avroid not found in dict  self.avscmap")
            self.create_avro_schid_instance(avscid)
            if "avroinstance" in self.avscmap[avscid]:
                self.__logger.debug(
                    "avroid and avroinstance found in dict  self.avscmap after creating with create_avro_schid_instance"
                )
                avroinstance = self.avscmap[avscid]["avroinstance"]
        self.__logger.debug("I will return the avroinstance: %s" % avroinstance)
        return avroinstance

    def create_avro_schid_instance(self, avscid):
        # global  self.avscmap
        avroinstance = None

        self.__logger.info("Creating avroinstance for avro-schemaid: %s" % (avscid))

        avsc = self.getavroschema(avscid)
        value_schema = avro.loads(json.dumps(avsc))
        # print("TEST:%s" % (value_schema))
        key_schema = avro.loads(
            '{"name": "schemaregistry", "type": "record", "fields": [{"name" : "schemaid", "type" : "long"}]}'
        )

        avroProducer = AvroProducer(
            {
                "bootstrap.servers": lib_pmgrpcd.OPTIONS.bsservers,
                "schema.registry.url": lib_pmgrpcd.OPTIONS.urlscreg,
                "schema.registry.ssl.ca.location": lib_pmgrpcd.OPTIONS.calocation,
                "security.protocol": lib_pmgrpcd.OPTIONS.secproto,
                "ssl.certificate.location": lib_pmgrpcd.OPTIONS.sslcertloc,
                "ssl.key.location": lib_pmgrpcd.OPTIONS.sslkeyloc,
                "ssl.ca.location": lib_pmgrpcd.OPTIONS.calocation,
            },
            default_key_schema=key_schema,
            default_value_schema=value_schema,
        )

        if avscid in self.avscmap:
            self.avscmap[avscid].update({"avroinstance": avroProducer})
        else:
            self.avscmap.update({avscid: {"avroinstance": avroProducer}})

        return avsc

    def getavroschema(self, avscid):
        # global  self.avscmap
        self.__logger.debug("In getavroschema with avscid: %s" % avscid)
        avsc = None
        if avscid in self.avscmap:
            if "avsc" in self.avscmap[avscid]:
                avsc = self.avscmap[avscid]["avsc"]
            else:
                self.loadavsc(avscid)
                if avscid in self.avscmap:
                    if "avsc" in self.avscmap[avscid]:
                        avsc = self.avscmap[avscid]["avsc"]
        else:
            # self.__logger.info("avsc not found in dict  self.avscmap")
            self.loadavsc(avscid)
            if avscid in self.avscmap:
                if "avsc" in self.avscmap[avscid]:
                    avsc = self.avscmap[avscid]["avsc"]
        return avsc

    # PMGRPCDLOG.info("PROTOPATH[" + telemetry_node + "]: " + protopath)

    def loadavsc(self, avscid):
        # global  self.avscmap
        self.__logger.debug("In loadavsc with avscid: %s" % avscid)
        avsc = None
        self.__logger.debug(
            "lib_pmgrpcd.OPTIONS.urlscreg: %s lib_pmgrpcd.OPTIONS.calocation: %s"
            % (lib_pmgrpcd.OPTIONS.urlscreg, lib_pmgrpcd.OPTIONS.calocation)
        )

        try:
            self.__logger.debug(
                "Instancing client (CachedSchemaRegistryClient) with avscid:%s url:%s ssl.ca.location:%s",
                avscid,
                lib_pmgrpcd.OPTIONS.urlscreg,
                lib_pmgrpcd.OPTIONS.calocation,
            )
            client = CachedSchemaRegistryClient(
                url=lib_pmgrpcd.OPTIONS.urlscreg,
                ca_location=lib_pmgrpcd.OPTIONS.calocation,
            )
        except Exception as e:
            self.__logger.info(
                "ERROR: load avro schema from schema-registry-server is failed on CachedSchemaRegistryClient on using method get_by_id()"
            )
            self.__logger.info("ERROR: %s" % (e))
            return avsc

        try:
            avsc = client.get_by_id(avscid)
        except Exception as e:
            self.__logger.info(
                "ERROR: load avro schema from schema-registry-server is failed on CachedSchemaRegistryClient on using method get_by_id()"
            )
            self.__logger.info("ERROR: %s" % (e))
            return avsc

        try:
            avsc_dict = json.loads(str(avsc))
        except Exception as e:
            self.__logger.info(
                "ERROR: json.loads of the avsc_str is faild to produce a dict"
            )
            self.__logger.info("ERROR: %s" % (e))
            return avsc

        self.__logger.info("SCHEMA_OF_ID(%s): %s" % (avscid, avsc_dict["name"]))

        # Query Schema-Registry
        # self.jsonmap = json.load(mapfile)
        if avscid in self.avscmap:
            self.__logger.debug(
                "Update  self.avscmap the existing record avscid (%s) with avroschema"
                % avscid
            )
            self.avscmap[avscid].update({"avsc": avsc_dict})
        else:
            self.__logger.debug(
                "Update  self.avscmap with new record avscid (%s) with avroschema"
                % avscid
            )
            self.avscmap.update({avscid: {"avsc": avsc_dict}})

        return avsc

    def delivery_report(self, err, msg):
        """ Called once for each message produced to indicate delivery result.
            Triggered by poll() or flush(). """
        if err:
            sys.stderr.write("%% Message failed delivery: %s\n" % err)
        elif lib_pmgrpcd.OPTIONS.debug:
            print("Message delivered to {} [{}]".format(msg.topic(), msg.partition()))

    def serialize(self, jsondata, topic, avscid, avroinstance):
        self.__logger.debug(
            "JSONDATA:%s\nTOPIC:%s\nAVSCID:%s\nAVROINSTANCE:%s\nSERIALIZELOG:%s"
            % (jsondata, topic, avscid, avroinstance, self.__logger)
        )
        if lib_pmgrpcd.OPTIONS.jsondatafile or lib_pmgrpcd.OPTIONS.rawdatafile:
            self.__logger.info(
                "JSONDATA:%s\nTOPIC:%s\nAVSCID:%s\nAVROINSTANCE:%s\nSERIALIZELOG:%s"
                % (jsondata, topic, avscid, avroinstance, self.__logger)
            )

        try:
            # https://github.com/confluentinc/confluent-kafka-python/issues/137

            result = avroinstance.produce(
                topic=topic, value=jsondata, callback=self.delivery_report
            )
            avroinstance.poll(0)

            if lib_pmgrpcd.OPTIONS.jsondatafile or lib_pmgrpcd.OPTIONS.rawdatafile:
                result = avroinstance.flush()

        except BufferError as e:
            print("[Exception avroinstance.produce BufferError]: %s" % (str(e)))
            self.__logger.debug(
                "[Exception avroinstance.produce BufferError]: see serializelog for details\n%s\n%s"
                % (json.dumps(jsondata, indent=2, sort_keys=True), str(e))
            )
            avroinstance.poll(10)
            result = avroinstance.produce(
                topic=topic,
                value=jsondata,
                key={"schemaid": avscid},
                callback=self.delivery_report,
            )
        except NotImplementedError as e:
            print("[Exception avroinstance.produce NotImplementedError]: %s" % (str(e)))
            self.__logger.debug(
                "[Exception avroinstance.produce NotImplementedError]: see serializelog for details\n%s\n%s"
                % (json.dumps(jsondata, indent=2, sort_keys=True), str(e))
            )
        except Exception as e:
            print("[Exception avroinstance.produce Exception]: %s" % (str(e)))
            self.__logger.debug(
                "[Exception avroinstance.produce Exception]: see serializelog for details\n%s\n%s"
                % (json.dumps(jsondata, indent=2, sort_keys=True), str(e))
            )

    def manually_serialize(self):
        PMGRPCDLOG.info(
            "manually serialize with  avscid (%s) and jsondatafile (%s)"
            % (lib_pmgrpcd.OPTIONS.avscid, lib_pmgrpcd.OPTIONS.jsondatafile)
        )
        avscid = int(lib_pmgrpcd.OPTIONS.avscid)
        avroinstance = self.getavro_schid_instance(avscid)
        with open(lib_pmgrpcd.OPTIONS.jsondatafile, "r") as jsondatahandler:
            jsondata = json.load(jsondatahandler)
        self.serialize(jsondata, lib_pmgrpcd.OPTIONS.topic, avscid, avroinstance)


def buildCtx():
    return KafkaAvroExporterContext()


def processor(state, data):
    return state.process_metric(data)


def manually_serialize():
    ctx = buildCtx()
    ctx.manually_serialize()


logQueue = multiprocessing.Queue()


class LoggingThread(threading.Thread):
    def __init__(self):
        super().__init__()

    def run(self):
        global logQueue
        while True:
            record = logQueue.get(block=True)
            if record is not None:
                lib_pmgrpcd.SERIALIZELOG.handle(record)
            else:
                return


logging_thread = LoggingThread()
logging_thread.start()


class KafkaAvroExporter(Exporter):
    def __init__(self):
        self.ws = WorkerSwarm(lib_pmgrpcd.OPTIONS.ProcessPool, buildCtx, processor)

        def term(sig, frame):
            self.ws.stop()
            self.ws.wait()

            global logQueue
            logQueue.put(None)
            global logging_thread
            logging_thread.join()

            if self.__prev_handler != 0:
                self.__prev_handler(sig, frame)

            os._exit(0)

        self.ws.start()

        self.__prev_handler = signal.getsignal(signal.SIGTERM)
        signal.signal(signal.SIGTERM, term)

    def process_metric(self, datajsonstring):
        self.ws.enqueue(datajsonstring)



#
#   pmacct (Promiscuous mode IP Accounting package)
#   pmacct is Copyright (C) 2003-2020 by Paolo Lucente
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
#   pmgrpcd and its components are Copyright (C) 2018-2020 by:
#
#   Matthias Arnold <matthias.arnold@swisscom.com>
#   Raphaël P. Barazzutti <raphael@barazzutti.net>
#   Juan Camilo Cardona <jccardona82@gmail.com>
#   Thomas Graf <thomas.graf@swisscom.com>
#   Paolo Lucente <paolo@pmacct.net>
#
# Imitates the generation of huawei telemetry.
# Use for testing.
import huawei_grpc_dialout_pb2_grpc
import grpc

# Dialout code
# Simple for now, it could get complicated if we need a more time based example.
class HuaweiDialOutClient():
    def __init__(self, server):
        self.server = server
        self.channel = grpc.insecure_channel(self.server)
        self.stub = huawei_grpc_dialout_pb2_grpc.gRPCDataserviceStub(self.channel)

    def send_data(self, data):
        self.rcv  = self.stub.dataPublish(data)


    def close(self):
        self.channel.close()


